# .
# active_check.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'digest/md5'
module Watobo
  module Core
    
    class ActiveCheck  < Watobo::Framework::Session    # Base Class for Passive Checks
      include Watobo::Core
      include Watobo::Utils
      
      attr :info
      attr :numChecks
      
      def addFinding(details_orig)
        details = details_orig.dup
        new_finding = Hash.new
        new_finding.update(@finding)
        t = Time.now
        #  puts "! addFinding: #{@info[:check_name]}"
        now = t.strftime("%m/%d/%Y@%H:%M:%S")  
        new_finding.update(details)
        new_finding[:tstamp] = now
        
        id_string = ''
        id_string << new_finding[:chat].request.url if new_finding[:chat]
        id_string << new_finding[:class] if new_finding[:class]  
        id_string << new_finding[:title]  if new_finding[:title]
        
        if id_string == '' then
          id_string = rand(10000)
        end
        #
        new_finding[:id] = Digest::MD5.hexdigest(id_string)
        
        new_finding[:module] = self.class.to_s
        
        if details[:debug] == true then
          puts "======="
          puts "Finding"
          puts "======="
          puts new_finding[:class]
          puts new_finding[:title]
          puts new_finding[:chat].request.first
          puts "---"
        end
        
        @project.addFinding(new_finding)
        
      end
      
      def reset()
        # should be overridden, if counters or status information are used!!!
      end
      
      def resetCounters()
        @numChecks = 0
        @counters = Hash.new
        @progress = 0
        reset()
      end
      
      def updateCounters(chat)
        
        @counters[chat.id] = getCheckCount(chat)
        
        @numChecks += @counters[chat.id]
        #puts "#{chat.id} : #{@numChecks}"
      end
      
      def urlParmNames(chat)
        pnames = chat.request.get_parm_names
        @project.settings[:ignore_parms].each do |p|
          pnames.delete(p)
        end
        return pnames
      end
      
      def postParmNames(chat)
        pnames = chat.request.post_parm_names
        @project.settings[:ignore_parms].each do |p|
          pnames.delete(p)
        end
        return pnames
      end
      
      def getCheckCount(chat)
        count = 0
        generateChecks(chat) do |check|
          count += 1
        end
        return count       
      end
      
      def enabled?
        @enabled
      end
      
      def enabled=(status)
        @enabled = status
      end
      
      def enable
        @enabled = true
      end
      
      def disable
        @enable = false
      end
      
      def generateChecks(chat)
        #return 0
      end
      
      def stop()
        @status = "stopped"
        #   puts "Threadlist: #{@check_threads.length}"
        @check_threads.each do |thr|
          #  puts "Stopping #{thr}"
          begin
            thr.stop if not thr.stop?
          rescue
            puts "could not stop thread #{thr}"
            
          end
        end
      end
      
      def run()
        @status = "ready"
        @check_threads.each do |thr|
          begin
            
            if thr.alive? and thr.stop?
              thr.run 
            end
          rescue => bang
            puts "could not start thread #{thr}"
          end

          
        end
      end
      
      def runLogin()
        @project.runLogin()
      end
      
      # +++ run_checks  +++
      # + function: wrapper function for doRequest(r). Needed for additional checks like smartchecks.
      #
      def run_checks(chat, prefs)
        reset() # reset variables first
        count = 0
        puts "* run_checks [#{chat.id}]"
        do_test(chat, prefs) { |request, response|
          count +=1
          @project.settings[:logout_signatures].each do |logout_pattern| 
            
            if response 
              response.each do |line|
                if line =~ /#{logout_pattern}/ then
                  puts "!!! GOT LOGGED OUT !!! [#{chat.id}]"    
                  #  @project.sessionMgr.runLogin()
                  count = -1
                end
              end
            end
          end
          break if count < 0
        }        
        return count
      end
      
      def do_test(chat, prefs)
        
        @check_threads.clear
        #  puts "starting generator"
        
        
        generateChecks(chat) do |check|
          @check_threads.push Thread.new(Thread.current) { |master|
            @project.numRunningChecks += 1
            begin
              request, response = check.call
              yield request, response
            rescue => bang
              puts "!!!ERROR: running check"
              puts bang
              #puts bang.backtrace
            ensure
              @progress += 1
              @project.numRunningChecks -= 1
              master.run # tell master to continue
            end
            prefs[:progress_display].increment(1) if prefs[:progress_display]
          }
          while  @status == "stopped" or @project.numRunningChecks > @project.settings[:max_parallel_checks]
            sleep(1) 
          end
        end
        checks_running = true
        while checks_running do
          checks_running = false
          @check_threads.each do |t|
            checks_running = true if t.alive?
          end
        end
        # puts "#{@info[:check_name]} finished (#{@progress}/#{@numChecks})"
      end
      
      def initialize(project)
        @project = project
        super(project)
        @enabled = true
        @status = "ready"
        @counters = Hash.new
        @numChecks = 0
        @progress = 0
        @check_threads = []
        
        @info = {
          :check_name => '',    # name of check which briefly describes functionality, will be used for tree and progress views
          :check_group => 'Misc',   # groupname of check, will be used to group checks, e.g. :Generic, SAP, :Enumeration 
          :description => '',   # description of checkfunction
          :author => "not modified", # author of check
          :version => "unversioned",   # check version
          :target => nil               # reserved
          
        }
        
        @finding = {
          :title => 'untitled',          # [String] title name, used for finding tree
          :check_pattern => nil,         # [String] regex of vulnerability check if possible, will be used for highlighting
          :proof_pattern => nil,         # [String] regex of finding proof if possible, will be used for highlighting
          :threat => '',        # threat of vulnerability, e.g. loss of information
          :measure => '',       # measure 
          :class => "undefined",# [String] vulnerability class, e.g. Stored XSS, SQL-Injection, ...
          :subclass => nil,     # reserved
          :type => FINDING_TYPE_UNDEFINED,         # FINDING_TYPE_HINT, FINDING_TYPE_INFO, FINDING_TYPE_VULN 
          :chat => nil,         # related chat must be linked
          :rating=> VULN_RATING_UNDEFINED,  # 
          :cvss => "n/a",       # CVSS Base Vector
          :icon => nil,     # Icon Type
          :timestamp => nil         # timestamp 
        }
        
      end
    end
  end
end
