# .
# fuzz_gen.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Core
    
    # base class for generators
    class FuzzGenerator
      attr :numTotal
      
      def run(imr) # imr (Intermediate Results)
        
        result = imr ? imr : Hash.new
        
        generate do |value|
          rv = value
          @action_list.each do |p|
            rv = p.call(rv)
          end
          result[@gen_name] = rv
          if @next_gen then
            @next_gen.run(result) do |cvalue|
              yield cvalue
            end
          else
            yield result
          end
        end
      end
      
      def addAction(proc)
        @action_list.push proc
      end
      
      def generate
        return true
      end
      
      def add2Chain(generator)
        @next = generator
      end
      
      
      def initialize(gen_name)
        @action_list = []
        @next_gen = nil
        @gen_name = gen_name
        @numTotal = 0
      end
      
    end
    
    class FuzzFile < FuzzGenerator
      def generate
      return if not @filename
      fh = File.open(@filename)
      fh.each_line do |line|
        yield line.chomp!
      end
    end
     
     def initialize(marker, filename)
       super(marker)
       if File.exists?(filename) then
         @filename = filename
         count = 0
         File.open(filename) do |fh|
           
           fh.each_line do |l|
             count += 1
           end
         end
         @numTotal = count
       else
         @numTotal = 0
       end
         
       
     end
      
    end
    
    
    class FuzzList < FuzzGenerator
      def generate
        @list.each do |item|
          yield item
        end
      end
    def initialize(marker, list)
      @list = list
      super(marker)
      @numTotal = @list.length
    end
    end
    
    
    class FuzzCounter < FuzzGenerator
      def generate
        return false if @start.nil?
        return false if @stop.nil? and @count.nil?
        
        if @stop == 0 and @count > 0 then
          @stop = @start + @count
        end
        
        return 0 if @start == @stop
        
        @start.step(@stop, @step) do |i| 
          yield i 
        end
        
        
      end
      
      def initialize(name, prefs)
        super(name)
        @start = prefs[:start]
        @stop =  prefs[:stop]  
        
        @count = prefs[:count] ? prefs[:count] : 0 
        @step = ( prefs[:step] and prefs[:step] != 0 ) ? prefs[:step] : 1
        
        if @stop < @start and @step > 0 then
          @step = @step * -1
        end
        
        @numTotal = (( @stop - @start ) / @step).abs
        
      end
      
    end
    
  end
end