# .
# scanner.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Core
    
    class OnlineCheck < Watobo::Framework::Session
      def initialize(project)
        super(project)
      end
      
      
      def isOnline?(chat)
        request = []
        request << "GET #{chat.request.url} HTTP/1.0\r\n"
        request << "Host: #{chat.request.site}\r\n"
        request << "\r\n"
        
        request.extend Watobo::Parser::Web10
        request.extend Watobo::Framework::Shaper::Web10
        begin
          site = chat.request.site
          puts "test if #{site} is online ..."
          puts request
          test_req, test_resp = self.doRequest(request, nil)
        
          if test_req.nil? then
            puts "Site #{site} is offline"
            return false
          end
          
        rescue => bang
          puts "!Error: Online-Check"
          puts "Site #{site} is offline"
          return false
        end
        puts "Site #{site} is online!"
        return true
      end
    end
    
    class Scanner
      attr :numTotalChecks
      attr :progress
      
      def running?()
        @status_running  
      end
      
      def stop()
        puts "!!! Scan Stopped !!!"
        @status_running = false
        @module_list.each do |mod|
          mod.stop()
        end
      end
      
      def siteAlive?(chat)
        site = chat.request.site
        if @sites_online.has_key?(site) then
          return @sites_online[site] ? true : false
        end
        if @onlineCheck.isOnline?(chat) then
          puts "Site #{site} is online"
          @sites_online[site] = true
          return true
        else
          #    puts "Site #{site} is offline"
          @sites_online[site] = false
          return false
        end
      end
      
      def continue()
        puts "!!! Scan Running !!!"
        @module_list.each do |mod|
          mod.run()
        end
        @status_running = true
      end
      
      def updateProgress(test_module, progress_step)
        @progress += progress_step
        @progress_display.progress = @progress if @progress_display
      end
      
      def run( prefs ) 
        @status_running = true
        @sites_online.clear
        @progress_displays = prefs[:progress_displays]
        @interface.logger("Starting Scan ...")
        @module_list.each do |mod|
          
          if not mod.enabled? then
            @interface.logger("skipping checks: #{mod.info[:check_name]}")
            next
          end
          @thread_list[mod] = []
          @interface.logger("running checks: #{mod.info[:check_name]}")
          @chat_list.each do |chat|
            if siteAlive?(chat) then
              @thread_list[mod].push Thread.new(chat) { |this_chat|
                # puts "+++ new thread #{@thread_list.length.to_s}"
                begin
                  rcode = mod.run_checks(this_chat, :progress_display => @progress_displays[mod])
                  #puts "---rcode: " + rcode.to_s
                  if rcode >= 0 then
                    # puts "sent #{rcode} requests on chat [#{this_chat.id}]"
                  else
                    puts "!!! RELOGIN !!!" 
                    mod.runLogin()
                    mod.updateCounters(this_chat)
                    mod.run_checks(this_chat, :progress_display => @progress_displays[mod])
                  end
                  
                rescue => r
                  puts "!!!ERROR: Starting scan for module #{mod.info[:check_name]}"
                  print "\nstopped working on chat(#{chat.id}): #{r}\n"
                  puts r.backtrace
                end              
              } 
            else
              @interface.logger("Site #{chat.request.site} unreachable!")
            end
          end
          
          
        end
        scan_running = true
        while scan_running do
          scan_running = false
          sleep(1)
          @thread_list.each do |mod, tlist|
            mod_active = false
            tlist.each do |thr|
              if thr.alive? then
                scan_running = true 
                mod_active = true
              end
            end
            if not mod_active then
              @interface.logger("finished #{mod.info[:check_name]}")
              @thread_list.delete(mod)
            end
            
          end
        end
        return 0
      end
      
      def initialize(project, interface, module_list, chat_list)
        
        @numTotalChecks = 0
        @project = project
        @module_list = module_list
        @chat_list = chat_list
        @thread_list = Hash.new
        @progress = 0
        @interface = interface
        @sites_online = Hash.new
        
        @onlineCheck = OnlineCheck.new(@project)
        
        @module_list.uniq.each do |m|
          m.resetCounters()
          puts "* updating counters for module #{m.info[:check_name]}"
          puts "* updating #{@chat_list.length} chats"
          @chat_list.each do |chat|
            
            m.updateCounters(chat)
          end
          @numTotalChecks += m.numChecks
        end
        
        
        
      end
    end
    
  end
end