# .
# dashboard.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Gui
    
    class ScanProgressFrame < FXVerticalFrame
      attr :progress_bars
      attr :scan_status
      
      def addProgressInfo(target, active_module)
        begin
          frame = FXVerticalFrame.new(target, :opts => LAYOUT_FILL_X|FRAME_GROOVE|LAYOUT_TOP)
          
          label = active_module.info[:check_name]
          
          l = FXLabel.new(frame, label, :opts => LAYOUT_LEFT)
          
          #   puts l
          pbar = FXProgressBar.new(frame, nil, 0, LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_HORIZONTAL)
          pbar.progress = 0
          pbar.total = active_module.numChecks
          puts "#{active_module.info[:check_name]} has #{active_module.numChecks} Checks"
          pbar.barColor = 0
          if active_module.numChecks == 0 then
            pbar.barColor = 'grey'
          else
          pbar.barColor = FXRGB(255,0,0)
          end 
          
          frame.create
          
          @progress_bars[active_module] = pbar
          
        rescue => bang
          puts "!!!ERROR: could not add progress info"
          puts bang
        end
        
      end
      
      def setup(modules)
        @progress_frame.each_child do |child|
          @progress_frame.removeChild(child)
        end
        
        @progress_bars = Hash.new
        modules.each do |mod|
          
          addProgressInfo(@progress_frame, mod) if mod.enabled?
        end
        #main.create
        @progress_frame.recalc
        @progress_frame.update
        
        
      end
      
      def initialize(owner, opts)
        super(owner, opts)
        
        #   frame = FXScrollWindow.new(self, SCROLLERS_NORMAL|LAYOUT_FILL_X|LAYOUT_FILL_Y) 
        frame = FXScrollWindow.new(self, SCROLLERS_NORMAL|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        info_container = FXVerticalFrame.new(frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
        frame = FXHorizontalFrame.new(info_container, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Scan-Status:")
        
        @scan_status = FXLabel.new(frame, "N/A")
        @progress_frame = FXVerticalFrame.new(info_container, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
        #FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
        #FXLabel.new(@main, "No Information Available")
      end
    end
    
    class ProjectInfo < FXVerticalFrame
      def update(project)
        if project then
        @project_name.text = project.settings[:project_name]
        @session_name.text = project.settings[:session_name]
        @project_path.text = project.settings[:project_path]
        @session_path.text = project.settings[:session_path]
        
        @number_active_checks.text = project.active_modules.length.to_s
        @number_passive_checks = project.passive_modules.length.to_s
        @number_total_chats.text = project.chat_list.length.to_s
        end
        
      end
      
      def initialize(owner, opts)
        super(owner, opts)
        frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Project:")
        @project_name = FXLabel.new(frame,"-")
        
        frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Session:")
        @session_name = FXLabel.new(frame,"-")
        
        frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Project Path:")
        @project_path = FXLabel.new(frame,"-")
        
        frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Session Path:")
        @session_path = FXLabel.new(frame,"-")
        
        frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Number available ActiveChecks:")
        @number_active_checks = FXLabel.new(frame,"-")
        
        frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Number of PassiveChecks:")
        @number_passive_checks = FXLabel.new(frame,"-")
        
        frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Number Current Chats:")
        @number_total_chats = FXLabel.new(frame,"-")
        
      #  frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
      #  FXLabel.new(frame, "Number Critical Findings:")
      #  @number_critical_findings = FXLabel.new(frame,"-")
        
      #  frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
      #  FXLabel.new(frame, "Number High Findings:")
      #  @number_high_findings = FXLabel.new(frame,"-")
        
      #  frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
      #  FXLabel.new(frame, "Number Medium Findings:")
      #  @number_medium_findings = FXLabel.new(frame,"-")
        
      #  frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
      #  FXLabel.new(frame, "Number Low Findings:")
      #  @number_low_findings = FXLabel.new(frame,"-")
      end
    end
    
    
    class Dashboard < FXVerticalFrame
      
      def setupScanProgressFrame(scan_modules)
        @scan_progress_frame.setup(scan_modules)
      end
      
      def setScanStatus(status)
        @scan_progress_frame.scan_status.text = status
      end
      
      def progressDisplays()
        return @scan_progress_frame.progress_bars()
      end
      
      def updateProjectInfo(project)
        @project = project
        @project_info_frame.update(@project)
      end
      
      def update_status(check_module, progress_index)
        if @module_list.has_key?(check_module) then
          #puts "updating status window"
          pbar = @module_list[check_module][:progress]
          pbar.total = @project.chat_list.length-1
          pbar.progress = progress_index
          if progress_index == pbar.total
            
            pbar.barColor=FXRGB(0,255,0)
          end
        else
          puts "check_module not found in dashboard"
        end
      end
      
      def setup_status_bars(frame, module_list)
        
        module_list.each do |m|
          
          dummy = FXVerticalFrame.new(frame,LAYOUT_FILL_X|FRAME_GROOVE)
          dummy.create 
          
          label = "undefined"
          begin
            
            label = m.check_name
          rescue => bang
            #  puts "no check name defined"
            # puts bang
          end
          # puts "."
          l = FXLabel.new(dummy, label, :opts => LAYOUT_LEFT)
          l.create
          #   puts l
          pbar = FXProgressBar.new(dummy, nil, 0, LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_HORIZONTAL)
          pbar.create
          
          pbar.progress = 0
          pbar.total = @project.chat_list.length-1
          pbar.barColor=0
          pbar.barColor=FXRGB(255,0,0)
          @module_list[m] = {
            :progress => pbar,
          }
          
        end
        
      end
      
      
      
      def initialize(parent)
        begin
          
          super(parent, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN)
          db_title = FXLabel.new(self, "DASHBOARD", :opts => LAYOUT_LEFT)
          @project = nil
          
          @tabBook = FXTabBook.new(self, nil, 0, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
          
          FXTabItem.new(@tabBook, "Project Information", nil)
          @project_info_frame = ProjectInfo.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
          
          FXTabItem.new(@tabBook, "Scan Information", nil)
          @scan_progress_frame = ScanProgressFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
          
          @tabBook.connect(SEL_COMMAND) do |sender, sel, tabItem|
            
            case tabItem.to_i
              when 0
              #  puts "Login Script Selected"
              @project_info_frame.update(@project)
              when 1
              # puts "Session IDs Selected"
              
              when 2
              # 
            end
          end
          
        rescue => bang
          puts "Error creating dashboard :("
          puts bang
        end
      end
    end
  end
end