# .
# findings_tree.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
#require 'qcustomize.rb'

module Watobo
  module Gui
    class FindingsTree < FXTreeList
      attr_accessor :project
      
      include Watobo::Core
      
      def refresh(project)
        
      end
      
      def hidden?(finding)
        
        @interface.default_settings[:domain_filters].each do |domainfilter|
          site = finding.request.site
          if site =~ /#{domainfilter}/ then
            @filtered_domains[site] = domainfilter
            
            return true
            
          end
        end
        false
      end
      
      def reload()
        self.clearItems
        
        @project.findings.each_value do |finding|
          addFinding(finding)
        end
        #        @findings_tree.expandTree(project_leaf)
        @interface.updateRequestTable(@project)
      end
      
      
      
      
      def hideDomain(domain_filter)
        @interface.default_settings[:domain_filters].push domain_filter
        @interface.updateTreeLists  
      end
      
      def addFinding(finding)
        begin
          site = nil
          if not hidden?(finding) then
            site = self.findItem(finding.request.site, nil,SEARCH_FORWARD|SEARCH_IGNORECASE)
            
            if not site then
              # found new site
              site = self.appendItem(nil, finding.request.site, ICON_PROJECT, ICON_PROJECT)
              self.appendItem(site, "Vulnerabilities", ICON_VULN, ICON_VULN)
              self.appendItem(site, "Hints", ICON_HINTS, ICON_HINTS)
              self.appendItem(site, "Info", ICON_INFO, ICON_INFO)
              #site = @findings_tree.moveItem(project.first,project,site)
              self.setItemData(site, :item_type_site)
              
            end
            
            
            finding_type=""
            
            case finding.info[:type]
              when FINDING_TYPE_INFO
              finding_type = "Info"
              icon = ICON_INFO_INFO
              
              when FINDING_TYPE_HINT
              finding_type = "Hints"
              icon = ICON_HINTS_INFO
              
              
              when FINDING_TYPE_VULN
              finding_type = "Vulnerabilities"
              if finding.info[:rating] == VULN_RATING_LOW
                icon = ICON_VULN_LOW
              end
              if finding.info[:rating] == VULN_RATING_MEDIUM
                icon = ICON_VULN_MEDIUM
              end
              if finding.info[:rating] == VULN_RATING_HIGH
                icon=ICON_VULN_HIGH
              end
              if finding.info[:rating] == VULN_RATING_CRITICAL
                icon=ICON_VULN_CRITICAL
              end
            end
            
            
            
            sub_tree = self.findItem(finding_type, site, SEARCH_FORWARD|SEARCH_IGNORECASE|SEARCH_NOWRAP)
            if sub_tree and sub_tree.parent == site and finding.info[:class]
              
              class_item = self.findItem(finding.info[:class], sub_tree, SEARCH_FORWARD|SEARCH_IGNORECASE|SEARCH_NOWRAP)
              if not class_item or class_item.parent != sub_tree
                class_item = self.appendItem(sub_tree, finding.info[:class], icon, icon)
              end
              title_item = self.findItem(finding.info[:title], class_item, SEARCH_FORWARD|SEARCH_IGNORECASE|SEARCH_NOWRAP)
              if not title_item or title_item.parent != class_item
                title_item = self.appendItem(class_item, finding.info[:title], nil, nil)
                self.setItemData(title_item, :title )
                # puts finding.info[:title]
              end
              #   puts title_item
              request_item = self.findItem(finding.request.resource, title_item, SEARCH_FORWARD|SEARCH_IGNORECASE|SEARCH_NOWRAP)
              if not request_item or request_item.parent != title_item
                text = "/" + finding.request.resource
                request_item = self.appendItem(title_item, text)
                self.setItemData(request_item, finding)
              end
            end
            
          end
        rescue => bang
          puts "!ERROR: could not add finding to tree"
          puts bang
        end
      end
      
      
      
      def initialize(parent, interface, project)
        @project = project
        @interface = interface
        @parent = parent
        super(parent, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_RIGHT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES|TREELIST_EXTENDEDSELECT)
        
        @filtered_domains = Hash.new # domains which already have been filtered
        
        self.connect(SEL_COMMAND) do |sender, sel, item|
          if self.itemLeaf?(item)
            getApp().beginWaitCursor do
              begin
                if item.data
                  if item.data.class.to_s =~ /Qfinding/
                    @interface.show_vuln(item.data)                  
                  end
                end
              rescue => bang
                puts "!!! Error: could not show selected finding"
                puts bang
              end
            end
          elsif item.data == :title then
            @interface.show_vuln(item.first.data) if item.first.data          
          end
        end
        
        self.connect(SEL_DOUBLECLICKED) do |sender, sel, item|
          if self.itemLeaf?(item)           
              begin
                  if item.data.class.to_s =~ /Qfinding/i then
                    #TODO: show vulnerability details
                     @interface.showFindingInfo(item.data)
                   else
                     puts item.data.class.to_s
                  end
              rescue => bang
                puts "!!! Error: could not show selected finding"
                puts bang
            end
          end          
        end
        
        self.connect(SEL_RIGHTBUTTONRELEASE) do |sender, sel, event|          
          unless event.moved?
            item = sender.getItemAt(event.win_x, event.win_y)
            
            #unless item.nil?
            
            FXMenuPane.new(self) do |menu_pane|
              data = item ? self.getItemData(item) : nil 
              #
              # DOMAIN MENU
              if data == :item_type_site then
                
                
                subdom_filters = []
                subdom_filters.push item.text
                dummy = item.text
                
                item.text.split(/\./).each do |subdom|
                  if dummy =~ /(^#{subdom})\.(.*)/ then
                    subdom_filters.push '.*'+$2
                    #  puts subdom_filters.length.to_s
                    dummy.gsub!(/#{subdom}./,'')
                  end
                end
                # puts "Excluded Site: #{item.text}"
                submenu = FXMenuPane.new(self) do |domain_menu|
                  
                  subdom_filters.each do |filter|
                    hide_domain = FXMenuCommand.new(domain_menu, "#{filter}" )
                    hide_domain.connect(SEL_COMMAND) {
                      hideDomain(filter)
                    }
                  end
                end
                FXMenuCascade.new(menu_pane, "Hide Domain", nil, submenu)
                
                #
                # FINDINGS MENU
              elsif data.class.to_s == "Qfinding" then
                doManual = FXMenuCommand.new(menu_pane, "Manual Request.." )
                doManual.connect(SEL_COMMAND) {
                  @interface.open_manual_request_toolkit(item.data)
                  
                  
                }
                info = FXMenuCommand.new(menu_pane, "Details..." )
                info.connect(SEL_COMMAND) {
                  @interface.showFindingInfo(item.data)}
              end
              #  refresh = FXMenuCommand.new(menu_pane, "Refresh" )
              submenu = FXMenuPane.new(self) do |domain_menu|
                
                @filtered_domains.each do |domain, filter|
                  hide_domain = FXMenuCommand.new(domain_menu, "#{domain}" )
                  hide_domain.connect(SEL_COMMAND) {
                    @interface.default_settings[:domain_filters].delete(filter)
                    @filtered_domains.clear
                    @interface.updateTreeLists
                  }
                end
              end
              FXMenuCascade.new(menu_pane, "Unhide Domains", nil, submenu)
              menu_pane.create
              menu_pane.popup(nil, event.root_x, event.root_y)
              
              
              app.runModalWhileShown(menu_pane)
              
              # end
            end
          end
        end
      end
    end
    # namespace end
  end
end