# .
# fuzzer_gui.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/qRequest.rb'
require 'lib/qShapers.rb'
require 'lib/guiUtils.rb'
require 'lib/qProject.rb'
require 'lib/chatviewer.rb'
require 'lib/requesteditor.rb'
require 'lib/core/scanner'
require 'lib/core/fuzz_gen'

module Watobo
  module Gui
    
    class RequestSender < Watobo::Framework::Session
      def initialize(project)
        super(project)
      end
      
      def runLogin(prefs)
        @project.loginscript_ids.each do |chatid|
          
          test_req = @project.getChat(chatid).copyRequest
          request,response = doRequest(test_req,:update_sids => true, :update_session => true, :update_contentlength => true)
        end
      end
      
      
      def sendRequest(new_request, prefs)
        
        if prefs
          if prefs[:update_session ] == true and prefs[:run_login ] == true
            runLogin(prefs)
          end
        end
        new_request.extend Watobo::Parser::Web10
        new_request.extend Watobo::Framework::Shaper::Web10
        test_req, test_resp = self.doRequest(new_request, prefs)
        #begin
        #  if prefs[:log_chat ] == true
        #    chat = Qchat.new(new_request, test_resp, @project.last_chatid + 1)
        #    @project.addChat(chat)
        #  end
        #rescue => bang
        #  puts bang
        #end
        return test_req, test_resp
      end
    end
    
    class FuzzEditor < RequestEditor
      
      def highlight(pattern)
        sindex = nil
        eindex = nil
        
        dummy = @textbox.to_s
        @textbox.setText(dummy)
        
        pos = 0
        @pattern_matches.clear
        
        loop do
          #  sindex = []
          #  eindex = []
          sindex, eindex = @textbox.findText(pattern, pos, SEARCH_EXACT|SEARCH_IGNORECASE|SEARCH_FORWARD)
          #   puts sindex
          sindex, eindex = @textbox.findText(pattern, pos, :flags => SEARCH_REGEX|SEARCH_IGNORECASE|SEARCH_FORWARD) if not sindex
          #  puts sindex
          sindex, eindex = @textbox.findText(Regexp.quote(pattern), pos, :flags => SEARCH_REGEX|SEARCH_IGNORECASE|SEARCH_FORWARD) if not sindex
          
          break if not sindex or sindex.length == 0
          
          
          pos = eindex.last+1
          
          sindex.length.times do |i|
            start = sindex[i]
            len = eindex[i] - sindex[i]
            @pattern_matches.push [ start, len] if start >= 0
            
          end
          
          break if sindex.last < 0
          #   puts sindex
          
        end
        
        # puts "* found pattern #{pattern} #{@pattern_matches.length} times"
        @markers = @interface.listMarkers()
        @pattern_matches.each do |start, len|
          
          style = @style
          match = @textbox.to_s.slice(start, len)
          match.gsub!(/%%/, '')
          style = @markers.include?(match) ? 1 : 2
          
          begin
            @textbox.changeStyle(start, len, style)
          rescue => bang
            puts "outch"
            puts bang
          end
        end
        return @pattern_matches
      end
      
      
      
      def parseRequest(fuzzels)
        new_request = nil
        if fuzzels then
          new_request = @textbox.to_s
          fuzzels.each do |marker, value|
            new_request.gsub!(/%%#{marker}%%/, value.to_s)
          end          
        end
        
        result = new_request.split(/\n/)
        result.each do |l|
          l << "\r\n"
        end
        
        return result 
      end
      
      
      def updateMarkers(markers)
        @markers = markers
      end
      
      def initialize(owner, interface, opts)
        super(owner, opts)
        @interface = interface
        @markers = []
        
      end
    end
    
    class FilterSettings < FXVerticalFrame
      
      def getMatchFilter()
        
      end
      
      def getCollector()  
        collector = case @filter_dt.value
          when 0
          index = @patternCombo.currentItem
          filter = @patternCombo.getItemText(index)
          proc { |response|
            matchlist = []
            response.each do |line|
              if line =~ /#{filter}/i then
                match = $2
                matchlist.push match
              end
            end
            return matchlist
          }
          
          when 1
          filter = @regex_filter_dt.value
          if filter != '' then
            proc { |response|
              matchlist = []
              response.each do |line|
                if line =~ /#{filter}/i then
                  match = $1
                  match = filter if not match
                  matchlist.push match
                end
              end
              return matchlist
            }
          else
            nil
          end
        end
      end
      
      
      def initialize(owner, sidpatterns)
        @sid_patterns = sidpatterns
        super(owner, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE )
        
        @filter_dt = FXDataTarget.new(0)
        group_box = FXGroupBox.new(self, "Filter", LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
        sid_frame = FXHorizontalFrame.new(group_box, :opts => LAYOUT_FILL_X)
        @sid_rb = FXRadioButton.new(sid_frame, "Session-ID", @filter_dt, FXDataTarget::ID_OPTION)
        
        regex_frame = FXHorizontalFrame.new(group_box, :opts => LAYOUT_FILL_X)
        @regex_rb = FXRadioButton.new(regex_frame, "Regex", @filter_dt, FXDataTarget::ID_OPTION + 1)
        @regex_filter_dt = FXDataTarget.new('')
        @regex_filter = FXTextField.new(regex_frame, 1, :target => @regex_filter_dt, :selector => FXDataTarget::ID_VALUE,
                                        :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN)
        
        # group_box = FXGroupBox.new(self, "Collection",LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0)
        # frame = FXVerticalFrame.new(group_box, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_GROOVE)
        # @collectionList = FXList.new(frame, :opts => LIST_EXTENDEDSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        @filter_dt.connect(SEL_COMMAND) { 
          @sid_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
          @regex_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        }
        
        if @sid_patterns then
          @patternCombo = FXComboBox.new(sid_frame, @sid_patterns.length, nil, 0,
                                         :opts => COMBOBOX_INSERT_LAST|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
          @patternCombo.numVisible = @sid_patterns.length
          @sid_patterns.each do |pattern|
            @patternCombo.appendItem(pattern, nil) 
          end
        else
          FXLabel.new(sid_frame, "NO SID PATTERNS DEFINED!")
          @sid_rb.disable
          @filter_dt.value = 1
          @regex_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        end
        
        
        #  group_box = FXGroupBox.new(self, "Test", LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
        #  @sample_count_dt = FXDataTarget.new('')
        #  frame = FXHorizontalFrame.new(group_box, :opts => LAYOUT_FILL_X)
        #  @sample_count = FXTextField.new(frame, 3, :target => @sample_count_dt, :selector => FXDataTarget::ID_VALUE,
        #                                  :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_COLUMN)
        #  button = FXButton.new(frame, "Sample", nil, nil, 0, FRAME_RAISED|FRAME_THICK)                                          
        #  button.connect(SEL_COMMAND) do  |sender, sel, ptr|
        #    @interface.startSample()
        #  end
        #  frame = FXHorizontalFrame.new(group_box, :opts => LAYOUT_FILL_X)
        #  FXLabel.new(frame, "Matched:")
        
        
      end
    end
    
    class FuzzerSourceSelect < FXVerticalFrame
      include Watobo::Core
      
      def onGeneratorChanged(sender, sel, selected)
        index = @genCombo.currentItem
        generator = @genCombo.getItemText(index)
        @gen_switcher.setCurrent(index)
        
      end
      
      def getMarkers()
        markers = []
        markers.push @marker.value
        return markers
      end
      
      def updateFields
        @file_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @gen_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @list_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @sourceFileText.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @cstartText.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @cstopText.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @cstepText.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @markerText.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        
      end
      
      def selectFile(sender, sel, ptr)
        filename = FXFileDialog.getOpenFilename(self, "Select Source File", @source_file.value)
        if filename != "" then
          if File.exists?(filename) then
            @source_file.value = filename
            @sourceFileText.handle(self, FXSEL(SEL_UPDATE, 0), nil)
          end
          
        end
      end
      
      def onValueSelect(sender, sel, selected)
        item = @valueList.currentItem
        if item >= 0 then
          @new_list_item_dt.value = @valueList.getItemText(item)
        end
      end
      
      def removeValue(sender, sel, ptr)
        item = @valueList.currentItem
        if item >= 0 then
          @valueList.removeItem(item)
        end
      end
      
      def addValue(sender, sel, ptr)
        if @new_list_item_dt.value != '' then
          index = @valueList.appendItem(@new_list_item_dt.value )
          @valueList.makeItemVisible(index)
          @new_list_item_dt.value = ''
          @new_list_item.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        end
      end
      
      def getGenerator()
        gen = case @source_dt.value
          when 0
          #puts "File Generator Selected"
          FuzzFile.new(@marker.value,
                       @source_file.value)
          when 1
          # counter selected
          FuzzCounter.new(@marker.value, 
                          :start => @cstart.value.to_i, 
                          :stop => @cstop.value.to_i, 
                          #:count => @ccount.value.to_i, 
                          :step => @cstep.value.to_i)
          when 2
          puts "List Generator Selected"
          list = []
          @valueList.each do |item|
            puts item
            list.push item.text
          end
          FuzzList.new(@marker.value, list)
        end
        return gen
      end
      
      def getEngine()
        index = @genCombo.currentItem
        puts index
        engine = case index
          when 0
          puts "Counter"
          FuzzCounter.new(@marker.value, 
                          :start => @cstart.value.to_i, 
                          :stop => @cstop.value.to_i, 
                          #:count => @ccount.value.to_i, 
                          :step => @cstep.value.to_i)
          when 1
          puts "Iterator"
        end
      end
      
      def initialize(owner, interface, opts)
        super(owner, opts)
        
        @interface = interface
        @marker = FXDataTarget.new("FUZZ")
        frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X)
        FXLabel.new(frame, "Marker:")
        @markerText = FXTextField.new(frame, 1, :target => @marker, :selector => FXDataTarget::ID_VALUE,
                                      :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN)
        @markerText.editable = false
        group_box = FXGroupBox.new(self, "Select Source", LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0)
        @source_dt = FXDataTarget.new(0)
        
        @source_dt.connect(SEL_COMMAND) do
          case @source_dt.value
            when 0
            puts "File"
            when 1
            puts "Generator"
            when 2
            puts "List"
          end
          @file_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
          @gen_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
          @list_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        end
        file_select_frame = FXHorizontalFrame.new(group_box, :opts => LAYOUT_FILL_X)
        @file_rb =  FXRadioButton.new(file_select_frame, "File", @source_dt, FXDataTarget::ID_OPTION)
        @source_file = FXDataTarget.new('')
        @sourceFileText = FXTextField.new(file_select_frame, 1, :target => @source_file, :selector => FXDataTarget::ID_VALUE,
                                          :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN)
        button = FXButton.new(file_select_frame, "Select")
        button.connect(SEL_COMMAND, method(:selectFile) )
        
        frame = FXHorizontalFrame.new(group_box, LAYOUT_FILL_X)
        @gen_rb = FXRadioButton.new(frame, "Counter", @source_dt, FXDataTarget::ID_OPTION + 1)
        
        
        @cstep = FXDataTarget.new(0)
        @cstepText = FXTextField.new(frame, 3, :target => @cstep, :selector => FXDataTarget::ID_VALUE,
                                     :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_COLUMN|LAYOUT_RIGHT)
        FXLabel.new(frame, "Step",nil, :opts => LAYOUT_RIGHT)                                  
        
        
        
        @cstop = FXDataTarget.new(0)
        @cstopText = FXTextField.new(frame, 3, :target => @cstop, :selector => FXDataTarget::ID_VALUE,
                                     :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_COLUMN|LAYOUT_RIGHT)
        FXLabel.new(frame, "Stop",nil, :opts => LAYOUT_RIGHT)
        
        
        @cstart = FXDataTarget.new(0)
        @cstartText = FXTextField.new(frame, 3, :target => @cstart, :selector => FXDataTarget::ID_VALUE,
                                      :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_COLUMN|LAYOUT_RIGHT)
        FXLabel.new(frame, "Start",nil, :opts => LAYOUT_RIGHT)
        
        frame = FXHorizontalFrame.new(group_box, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
        @list_rb = FXRadioButton.new(frame, "List", @source_dt, FXDataTarget::ID_OPTION + 2)
        list_frame = FXVerticalFrame.new(frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
        frame = FXHorizontalFrame.new(list_frame, :opts => LAYOUT_FILL_X, :padding => 0)
        
        @new_list_item_dt = FXDataTarget.new('')
        @new_list_item = FXTextField.new(frame, 10,
                                         :target => @new_list_item_dt, :selector => FXDataTarget::ID_VALUE,
                                         :opts => TEXTFIELD_NORMAL|LAYOUT_SIDE_LEFT|LAYOUT_FILL_X)
        #   FXLabel.new(frame, "Value: ")
        @addButton = FXButton.new(frame, "Add", nil, nil, 0, :opts => FRAME_RAISED|FRAME_THICK)
        @addButton.connect(SEL_COMMAND, method(:addValue))
        @remButton = FXButton.new(frame, "Remove", nil, nil, 0, :opts => FRAME_RAISED|FRAME_THICK)
        @remButton.connect(SEL_COMMAND, method(:removeValue))
        
        list_border = FXVerticalFrame.new(list_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_GROOVE, :padding => 0)
        @valueList = FXList.new(list_border, :opts => LIST_EXTENDEDSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        @valueList.numVisible = 4
        
        @valueList.connect(SEL_COMMAND, method(:onValueSelect))
        
        
        updateFields()
        
      end
    end
    
    
    
    class FuzzerGui < FXDialogBox
      
      include Watobo::Gui::Utils
      include Watobo::Core
      
      attr_accessor :fuzzels
      
      def onRequestReset(sender,sel,item)
        @requestEditor.setText(@request)
      end
      
      def listMarkers()
        markers = []
        markers.concat @sourceSelect.getMarkers()
        return markers
      end
      
      
      def saveCollection(sender, sel, ptr)
        puts "need more time for programming :("
      end
      
      def logger(message)
        t = Time.now  
        now = t.strftime("%m/%d/%Y @ %H:%M:%S")  
        @log_viewer.insertText(0, "#{now}: #{message}\n")
        
      end
      
      def onBtnQuickScan_UNUSED(sender, sel, item)
        dlg = Watobo::Gui::QuickScanDialog.new(self, "Select ActiveChecks", @project.active_modules, @project.settings[:active_policies], @project.settings[:active_policy])
        if dlg.execute != 0 then
          scan_modules = []
          scan_chats = []      
          scan_chats.push Qchat.new(@request, [""], 0)
          
          pblist = Hash.new # hash for storing progressbar information
          
          @project.settings[:active_policy] = dlg.active_policy
          @project.active_modules.each do |m|
            if m.enabled? then
              scan_modules.push m
              # setup progress bars
              pblist[m] = @pbar
            else
              puts "skipping #{m.info[:check_name]}..."
            end
          end
          scanner = Watobo::Core::Scanner.new(self, scan_modules, scan_chats)
          puts "Checks Total: #{scanner.numTotalChecks}"
          @pbar.total = scanner.numTotalChecks
          @pbar.progress = 0
          @pbar.barColor = FXRGB(255,0,0)
          
          Thread.new {
            begin
              scanner.run(:progress_displays => pblist)
              logger("scan finished")
              @pbar.total = 0
              @pbar.progress = 0
              @pbar.barColor = 'grey'
            rescue => bang
              puts bang
            end
          }
        end
        
        return 0
        
      end
      
      def startSample(count)
        
      end
      
      def startFuzzing()
        fuzz_threads = []
        fuzzGen = @sourceSelect.getGenerator()
        
        collector = @collectorSettings.getCollector()
        if fuzzGen and collector then
          logger("Started Fuzzing (#{fuzzGen.numTotal} Requests)")
          @pbar.total = fuzzGen.numTotal
          @pbar.progress = 0
          Thread.new {
            fuzzGen.run(nil) do |fuzzels|
              fuzz_request = @requestEditor.parseRequest(fuzzels)            
              
              fuzz_threads.push Thread.new(Thread.current, fuzz_request, fuzzels.dup) { |master, trequest, fuzz|
                @project.numRunningChecks += 1
                begin
                  request, response = @sender.sendRequest(trequest, nil)
                  collector.call(response).each do |item|
                    if item then
                      puts item
                      @collectionList.appendItem(item)
                      lastRowIndex = @matchTable.getNumRows
                      @matchTable.appendRows(1)   
                      @matchTable.setItemText(lastRowIndex, 0, fuzz.to_a.join('='))
                      @matchTable.getItem(lastRowIndex, 0).justify = FXTableItem::LEFT        
                      
                      @matchTable.setItemText(lastRowIndex, 1, item)
                      @matchTable.getItem(lastRowIndex, 1).justify = FXTableItem::LEFT
                    end
                  end
                rescue => bang
                  puts "!!!ERROR: running check"
                  puts bang
                  puts bang.backtrace
                ensure
                  
                  @project.numRunningChecks -= 1
                  master.run # tell master to continue
                end
                @pbar.increment(1)
              }
              while not @fuzzing_paused and @project.numRunningChecks > @project.settings[:max_parallel_checks]
                sleep(1) 
              end
              break if @fuzzing_paused
            end
            checks_running = true
            while checks_running do
              checks_running = false
              fuzz_threads.each do |t|
                t.kill if @fuzzing_paused
                checks_running = true if t.alive?
              end
            end
            logger("Finished Fuzzing")
          }
        else
          logger("Could not start fuzzing :(")
        end
      end
      
      
      def onStart(sender,sel,item)
        @request_viewer.setText('')
        @response_viewer.setText('')
        new_request = @req_builder.parseRequest
        logger("sending request") 
        
        Thread.new(new_request) { |test_request|
          
          @last_request, @last_response = @request_sender.sendRequest(test_request, 
                                                                      :run_login => @runLogin.checked?, 
                                                                      :update_session => @updateSession.checked?, 
                                                                      :update_contentlength => @updateContentLength.checked?,
                                                                      :log_chat => @logChat.checked?)
          if @request_sender.lasterror then 
            logger("ERROR: #{@request_sender.lasterror}");
          else
            logger("got answer")
          end
          @request_viewer.setText(@last_request)
          @response_viewer.setText(@last_response)
          
        }
      end
      
      
      
      
      
      def initialize(owner, project, chat)
        # Invoke base class initialize function first
        super(owner, "Fuzzer", :opts => DECOR_ALL,:width=>800, :height=>600)
        self.icon = ICON_FUZZER
        @project = project
        @sender = RequestSender.new(@project)
        @request = chat.request.dup
        @fuzzing_paused = false
        @fuzzing_started = false
        
        @fuzzels = FXDataTarget.new()
        
        mr_splitter = FXSplitter.new(self, LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_VERTICAL|SPLITTER_REVERSED|SPLITTER_TRACKING)
        # top = FXHorizontalFrame.new(mr_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_SIDE_BOTTOM)
        top_frame = FXVerticalFrame.new(mr_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y||LAYOUT_FIX_HEIGHT|LAYOUT_BOTTOM,:height => 500) 
        top_splitter = FXSplitter.new(top_frame, LAYOUT_FILL_X|SPLITTER_HORIZONTAL|LAYOUT_FILL_Y|SPLITTER_TRACKING)
        
        log_frame = FXVerticalFrame.new(mr_splitter, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM,:height => 100)
        
        #LAYOUT_FILL_X in combination with LAYOUT_FIX_WIDTH
        
        req_editor = FXVerticalFrame.new(top_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|FRAME_GROOVE,:width => 400, :height => 500)
        
        req_edit_header = FXHorizontalFrame.new(req_editor, :opts => LAYOUT_FILL_X)
        FXLabel.new(req_edit_header, "Request:" )
        req_viewer = FXVerticalFrame.new(req_editor, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding => 0)
        req_reset_button = FXButton.new(req_edit_header, "Reset", nil, nil, 0, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT)
        req_reset_button.connect(SEL_COMMAND, method(:onRequestReset))
        
        
        frame = FXHorizontalFrame.new(req_editor, :opts => LAYOUT_FILL_X|FRAME_GROOVE)
        @fuzz_button = FXButton.new(frame, "Start", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
        @fuzz_button.connect(SEL_COMMAND) {
          if @fuzzing_started then
            @fuzzing_paused = true
            @fuzz_button.text = "Start"
            @fuzzing_started = false
          else
            @fuzzing_started = true
            @fuzzing_paused = false
            @fuzz_button.text = "Cancel"
            startFuzzing()
          end
        }
        
        @pbar = FXProgressBar.new(frame, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_HORIZONTAL)
        
        @pbar.progress = 0
        @pbar.total = 0
        @pbar.barColor=0
        @pbar.barColor = 'grey' #FXRGB(255,0,0)
        @requestEditor = FuzzEditor.new(req_viewer, self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
        @requestEditor.setText(@request)
        
        
        fuzz_setup_frame = FXVerticalFrame.new(top_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE|LAYOUT_FIX_WIDTH, :width => 400)
        
        @tabBook = FXTabBook.new(fuzz_setup_frame, nil, 0, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
        
        FXTabItem.new(@tabBook, "Basic", nil)
        bframe = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        @sourceSelect = FuzzerSourceSelect.new(bframe, self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        
        FXTabItem.new(@tabBook, "Filter", nil)
        vframe = FXVerticalFrame.new(@tabBook, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        @collectorSettings = FilterSettings.new(vframe, @project.settings[:sid_patterns])
        
        FXTabItem.new(@tabBook, "Results", nil)
        frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        
        itb = FXTabBook.new(frame, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
        FXTabItem.new(itb, "Matches", nil)
        frame = FXVerticalFrame.new(itb, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        
        @matchTable = FXTable.new(frame, :opts => TABLE_COL_SIZABLE|TABLE_ROW_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y|TABLE_READONLY|LAYOUT_SIDE_TOP, :padding => 2)
        @matchTable.setTableSize(0, 2)
        @matchTable.visibleRows = 20
        @matchTable.rowHeader.width = 0
        #@matchTable.visibleColumns = @columns.length
        @matchTable.setColumnText( 0, "Value(s)" ) 
        @matchTable.setColumnText( 1, "Response Code" )
        
        FXTabItem.new(itb, "Collected Data", nil)
        frame = FXHorizontalFrame.new(itb, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        listframe = FXVerticalFrame.new(frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_GROOVE|FRAME_THICK, :padding => 0)
        @collectionList = FXList.new(listframe, :opts => LIST_EXTENDEDSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        
        
        log_frame_header = FXHorizontalFrame.new(log_frame, :opts => LAYOUT_FILL_X)
        FXLabel.new(log_frame_header, "Logs:" )
        log_text_frame = FXVerticalFrame.new(log_frame, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_GROOVE, :padding=>0)
        @log_viewer = FXText.new(log_text_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
        #--------------------------------------------------------------------------------
        
      end
      
    end
  end
end