# .
# hex_viewer.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Gui
    class HexViewer < FXHorizontalFrame
      def normalizeText(text)
        dummy = []
        begin
          text.headers.each do |h|
            dummy.push h.strip + "\r\n"
          end
          dummy.push "\r\n"
          dummy.push text.body
          dummy = dummy.join
        rescue => bang
          dummy = text
        end
        return dummy
      end
      
      def setText(raw_text)
        initTable()
        
        if raw_text and not raw_text.empty? then
          raw_text = normalizeText(raw_text)
          pos = 0
          col = 0
          row = @hexTable.getNumRows
          @hexTable.appendRows(1)
          @hexTable.rowHeader.setItemJustify(row, FXTableItem::LEFT)    
          @hexTable.setRowText(row, "%0.4X" % row.to_s)
          while pos < raw_text.length
            if pos > 0 and pos % 15 == 0 then
              
              @hexTable.setItemText(row, 15, raw_text[pos-15..pos].gsub(/[^[:print:]]/,'.'))
              @hexTable.getItem(row, 15).justify = FXTableItem::LEFT
              col = 0
              row = @hexTable.getNumRows
              @hexTable.appendRows(1)
              @hexTable.rowHeader.setItemJustify(row, FXTableItem::LEFT)        
              @hexTable.setRowText(row, "%0.4X" % row.to_s)
            end
            @hexTable.setItemText(row, col, "%0.2X" % raw_text[pos])
            @hexTable.getItem(row, col).justify = FXTableItem::LEFT
            pos += 1
            col += 1
          end
          @hexTable.setItemText(row, 15, raw_text[row*15..pos].gsub(/[^[:print:]]/,'.'))
              @hexTable.getItem(row, 15).justify = FXTableItem::LEFT
          
        end
      end
      
      def initialize(owner)
        super(owner, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
        sunken = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding => 0)
        @hexTable = FXTable.new(sunken, :opts => FRAME_SUNKEN|TABLE_COL_SIZABLE|TABLE_ROW_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y|TABLE_READONLY|LAYOUT_SIDE_TOP, :padding => 2)
        
        
      end
      
      private
      def initTable
        @hexTable.clearItems()
        @hexTable.setTableSize(0, 16)
        @hexTable.rowHeader.width = 50
        1.upto(15) do |i|       
          htext = "%X" % i
          @hexTable.setColumnText( i-1, htext)          
          @hexTable.setColumnWidth(i-1, 35)
        end  
        @hexTable.setColumnWidth(15, 115)
      end
    end
  end
end