# .
# sites_tree.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
#require 'qcustomize.rb'

module Watobo
  module Gui
    class SitesTree < FXTreeList
      attr_accessor :project
      
      def reload()
        self.clearItems
        
        @project.chat_list.each do |chat|
          addChat(chat)
        end
        
        @interface.updateRequestTable(@project)
      end
      
      
      def hidden?(chat)
        
        @interface.default_settings[:domain_filters].each do |domainfilter|
          site = chat.request.site
          if site =~ /#{domainfilter}/ then
            @filtered_domains[site] = domainfilter
            
            return true
            
          end
        end
        false
      end
      
      def hideDomain(domain_filter)
        @interface.default_settings[:domain_filters].push domain_filter
        @interface.updateTreeLists()  
      end
      
      def addChat(chat)
        
        return 0 if hidden?(chat)
        
        site = self.findItem(chat.request.site, nil,SEARCH_FORWARD|SEARCH_IGNORECASE)
        
        if not site then
          # found new site
          site = self.appendItem(nil, chat.request.site, @siteIcon, @siteIcon)
          #site = @findings_tree.moveItem(project.first,project,site)
          self.setItemData(site, :item_type_site)
          
        end
        
        folder_parent = site
        #  puts "ADD_REQUEST: #{site.text} #{chat.request.path} \r\n #{chat.request.first}"
        path = chat.request.path
        if path then
          #puts "Check Folder: #{chat.request.path} - #{chat.request.site}" if path =~ /jump/
          folders=path.split('/')
          folders.each do |folder_name|        
            #       puts "search for folder #{folder_name}"
            folder_item = self.findItem(folder_name,folder_parent,SEARCH_FORWARD)
            if not folder_item then
              
              folder_item = self.appendItem(folder_parent, folder_name, @folderIcon, @folderIcon)
              self.setItemData(folder_item, :item_type_folder)
              #puts "added folder #{folder_name} to #{folder_parent} for site #{chat.request.site}" if folder_name =~ /intern/
            end
            folder_parent = folder_item            
          end          
        end
        
        element=chat.request.element
        if element
          if not self.findItem(element,folder_parent,SEARCH_FORWARD)
            new_item=self.appendItem(folder_parent, element, @reqIcon, @reqIcon)
            self.setItemData(new_item, chat)
          end
          
        end
        # end
        
      end
      
      
      
      def initialize(parent, interface, project)
        @project = project
        @interface = interface
        @parent = parent
        super(parent, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_RIGHT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES|TREELIST_EXTENDEDSELECT)
        
        @projectIcon = ICON_PROJECT
        
        @folderIcon = ICON_FOLDER
        @reqIcon = ICON_REQUEST
        @siteIcon= ICON_SITE
        
        @filtered_domains = Hash.new # domains which already have been filtered
        
        #    session_leaf = self.appendItem(nil, @session_name, @projectIcon, @projectIcon)
        
        self.connect(SEL_COMMAND) do |sender, sel, item|
          if self.itemLeaf?(item)
            getApp().beginWaitCursor do
              begin
                if item.data
                  #if item.data.class.to_s =~ /Qchat/
                  @interface.show_chat(item.data)
                  #end
                  
                end
              rescue
                puts "!!! Error: could not show selected tree item"
              end
            end
          end
          
          
        end
        
        self.connect(SEL_RIGHTBUTTONRELEASE) do |sender, sel, event|
          exclude_site = nil
          unless event.moved?
            item = sender.getItemAt(event.win_x, event.win_y)
            
            unless item.nil?
              data = self.getItemData(item)
              FXMenuPane.new(self) do |menu_pane|
                
                if data == :item_type_site then
                  subdom_filters = []
                  subdom_filters.push item.text
                  dummy = item.text
                  
                  item.text.split(/\./).each do |subdom|
                    if dummy =~ /(^#{subdom})\.(.*)/ then
                      subdom_filters.push '.*'+$2
                      #  puts subdom_filters.length.to_s
                      dummy.gsub!(/#{subdom}./,'')
                    end
                  end
                  # puts "Excluded Site: #{item.text}"
                  submenu = FXMenuPane.new(self) do |domain_menu|
                    
                    subdom_filters.each do |filter|
                      hide_domain = FXMenuCommand.new(domain_menu, "#{filter}" )
                      hide_domain.connect(SEL_COMMAND) {
                        hideDomain(filter)
                      }
                    end
                  end
                  FXMenuCascade.new(menu_pane, "Hide Domain", nil, submenu)
                  
                elsif data.class.to_s == "Qchat" 
                  doManual = FXMenuCommand.new(menu_pane, "Manual Request.." )
                  
                  doManual.connect(SEL_COMMAND) {
                    if item.data
                      @interface.open_manual_request_toolkit(item.data)
                    end
                    
                  }
                end
                submenu = FXMenuPane.new(self) do |domain_menu|
                
                @filtered_domains.each do |domain, filter|
                  hide_domain = FXMenuCommand.new(domain_menu, "#{domain}" )
                  hide_domain.connect(SEL_COMMAND) {
                    @interface.default_settings[:domain_filters].delete(filter)
                    @filtered_domains.clear
                    @interface.updateTreeLists
                  }
                end
              end
              FXMenuCascade.new(menu_pane, "Unhide Domains", nil, submenu)
                menu_pane.create
                menu_pane.popup(nil, event.root_x, event.root_y)
                app.runModalWhileShown(menu_pane)
                
              end
            end
          end
        end
      end
    end
    # namespace end
  end
end