# .
# start_full_scan_dialog.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/gui/check_policy_menu'
require 'lib/gui/select_sites_frame'

module Watobo
  module Gui
    class FullScanDialog < FXDialogBox
      
      include Responder
      attr :active_policy
      
      def onAccept(sender, sel, event)
        
        @active_policy = @policyFrame.policy_name
        @project.active_modules.each do |c|
          c.enabled = @policyFrame.isElementChecked(c) ? true : false
        end
        
        @project.settings[:target_sites] = @selectSitesFrame.selectedSites()
        @project.settings[:proxy] = @selectedProxy
        @project.settings[:proxy_list] = @selectedProxyList
        @project.settings[:max_parallel_checks] = @max_par_checks.value.to_i
        @project.settings[:enable_smart_scan] = @enable_smart_scan.checkState
        
        @project.loginscript_ids = @loginscript_ids  
        @project.settings[:sid_patterns] = @sid_patterns
        @project.settings[:logout_signatures] = @logout_signatures
        
        
        getApp().stopModal(self, 1)
        self.hide()
        return 1
      end
      
      def setButtons(index)
        case index
          when 0 
          
          @nextButton.enable
          @backButton.disable
          @finishButton.disable
          
          when 1
          # select session screen
          @nextButton.enable
          @backButton.enable
          @finishButton.enable
          
          when 2
          @nextButton.disable
          @backButton.enable
          @finishButton.enable
          
        end
      end
      
      def startProxyDialog(sender, sel, ptr)
        proxy_dialog = Watobo::Gui::ProxyDialog.new(self, @selectedProxy, @selectedProxyList)
        if proxy_dialog.execute != 0 then
          @selectedProxyList = proxy_dialog.getProxyList
          @selectedProxy = proxy_dialog.proxy
          @forwardingProxy.value = @selectedProxy
        end
      end
      
      def startSessionManagementDialog(sender, sel, ptr)
        smdlg = SessionManagementDialog.new(self, @project)
        if smdlg.execute != 0 then
          
          @loginscript_ids = smdlg.getLoginScriptIds() 
          @sid_patterns = smdlg.getSidPatterns()
          @logout_signatures = smdlg.getLogoutSignatures()
          
        end
      end
      
      def initialize(parent, project, prefs)
        super(parent, "Start Full Scan", DECOR_TITLE|DECOR_BORDER, :width => 300, :height => 425)
        @project = project
        
        @selectedProxy = @project.settings[:proxy]
        @selectedProxyList = @project.settings[:proxy_list]
        
        @loginscript_ids = @project.loginscript_ids  
        @sid_patterns = @project.settings[:sid_patterns]
        @logout_signatures = @project.settings[:logout_signatures]
        
        
        
        FXMAPFUNC(SEL_COMMAND, ID_ACCEPT, :onAccept)
        
        base_frame = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
        
        @switcher = FXSwitcher.new(base_frame,LAYOUT_FILL_X|LAYOUT_FILL_Y)   
        
        @selectedSites = []
        @selectSitesFrame = SelectSitesFrame.new(@switcher, project, prefs)
        
        @policyBase = FXVerticalFrame.new(@switcher, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
        smf = FXHorizontalFrame.new(@policyBase, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_TOP|FRAME_GROOVE)
        FXLabel.new(smf, "Select Checks")
        
        @policyFrame = ChecksPolicyFrame.new(@policyBase, @project.active_modules, @project.settings[:active_policies], @project.settings[:active_policy])
        
        
        @advancedFrame = FXVerticalFrame.new(@switcher, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
        
        smf = FXHorizontalFrame.new(@advancedFrame, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_TOP)
        FXLabel.new(smf, "Advanced Settings")
        
        gbox_frame = FXGroupBox.new(@advancedFrame, "Scan Options ", LAYOUT_SIDE_RIGHT|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
        gbox = FXVerticalFrame.new(gbox_frame, :opts => LAYOUT_SIDE_TOP|PACK_UNIFORM_WIDTH)
        frame = FXHorizontalFrame.new(gbox, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_RIGHT)
        FXLabel.new(frame, "Max. Par. Request:")
        @max_par_checks = FXDataTarget.new(0)
        @max_par_checks.value = @project.settings[:max_parallel_checks]
        FXTextField.new(frame, 3, @max_par_checks, FXDataTarget::ID_VALUE, :opts => JUSTIFY_RIGHT|FRAME_GROOVE|FRAME_SUNKEN)
        
        
        @enable_smart_scan = FXCheckButton.new(gbox, "Enable Smart Scan ", nil, 0, JUSTIFY_RIGHT|JUSTIFY_TOP|ICON_AFTER_TEXT|LAYOUT_SIDE_RIGHT)
        @enable_smart_scan.checkState = @project.settings[:enable_smart_scan]
        
        gbox = FXGroupBox.new(@advancedFrame, "Excluded Chats",LAYOUT_SIDE_RIGHT|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
        @excludedChats = FXDataTarget.new('')
        @excludedChats.value = @project.excluded_chats.join(", ")
        FXTextField.new(gbox, 0, @excludedChats, FXDataTarget::ID_VALUE, :opts => LAYOUT_FILL_X|FRAME_GROOVE|FRAME_SUNKEN)
        
        frame = FXHorizontalFrame.new(gbox, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_RIGHT)
        button = FXButton.new(frame, "View/Edit" ,  nil, nil, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        button.enable
        button.connect(SEL_COMMAND) do |sender, sel, item|
          
        end
        @ignoreEqualChats = FXCheckButton.new(frame, "Ignore Similar Chats", nil, 0, ICON_AFTER_TEXT|LAYOUT_LEFT)
        @ignoreEqualChats.checkState = false
        #-------------------------- Proxy ---------------------------------#
        gbox = FXGroupBox.new(@advancedFrame, "Forwarding Proxy",LAYOUT_SIDE_RIGHT|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
        frame = FXHorizontalFrame.new(gbox, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_RIGHT)
        FXLabel.new(frame, "Current: ")
        @forwardingProxy = FXDataTarget.new('')
        FXTextField.new(frame, 0, @forwardingProxy, FXDataTarget::ID_VALUE, :opts => LAYOUT_FILL_X|FRAME_GROOVE|FRAME_SUNKEN)
        
        @forwardingProxy.value = @project.settings[:proxy]
        button = FXButton.new(frame, "View/Edit" ,  nil, nil, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        button.enable
        button.connect(SEL_COMMAND, method(:startProxyDialog))
        #-------------------------- Session Management ---------------------------------#
        gbox = FXGroupBox.new(@advancedFrame, "Session Management",LAYOUT_SIDE_RIGHT|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
        frame = FXHorizontalFrame.new(gbox, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_RIGHT)
        #FXLabel.new(frame, " ")
        button = FXButton.new(frame, "View/Edit" ,  nil, nil, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        button.enable
        button.connect(SEL_COMMAND, method(:startSessionManagementDialog))
        
        
        buttons_frame = FXHorizontalFrame.new(base_frame,
                                              :opts => LAYOUT_FILL_X|LAYOUT_SIDE_TOP)
        
        @finishButton = FXButton.new(buttons_frame, "Start" ,  nil, nil, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        @finishButton.disable
        @finishButton.connect(SEL_COMMAND) do |sender, sel, item|
          #self.handle(self, FXSEL(SEL_COMMAND, ID_CANCEL), nil)
          self.handle(self, FXSEL(SEL_COMMAND, ID_ACCEPT), nil)
        end 
        
        @nextButton = FXButton.new(buttons_frame, "Next" ,  nil, nil, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        @nextButton.enable
        @nextButton.connect(SEL_COMMAND) do |sender, sel, item|
          if @switcher.current < @switcher.numChildren-1
            @switcher.current = @switcher.current+1
            
          end
          setButtons(@switcher.current)
        end 
        
        @backButton = FXButton.new(buttons_frame, "Back" ,  nil, nil, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        @backButton.disable
        @backButton.connect(SEL_COMMAND) do |sender, sel, item|
          if @switcher.current > 0
            @switcher.current = @switcher.current-1
            
          end
          setButtons(@switcher.current)
        end 
        
        @cancelButton = FXButton.new(buttons_frame, "Cancel" ,
        :target => self, :selector => FXDialogBox::ID_CANCEL,
        :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        
        
      end
    end
  end
end
