# .
# httpparser.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/core/watobo_constants'

module Watobo
  module Parser
    
    module Web10
      include Watobo::Core
      
      def urlparms
        dummy = self.first.gsub(/\?+/,"?")
        parts = dummy.split('?')
        if parts[1] then
          if parts[1] =~ /(.*) HTTP/i then          
            return $1
          else
            return nil
          end
        else
          return nil
        end 
      end
      
      
      def resource
        dummy = self.first.gsub(/\?+/,"?")
        parts = dummy.split('?')
        parts[0].gsub!(/ HTTP\/(.*)/i,"")
        if parts[0] =~ /\w+ (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)(.*)/i then          
          return $2
        else
          return nil
        end        
      end
      
      def method
        dummy = self.first
        if dummy =~ /(\w*) http/i then          
          return $1
        else
          return nil
        end
      end
      
      def path
        begin
          dummy = self.first.gsub(/\?+/,"?")
          parts = dummy.split('?')
          
          parts[0].gsub!(/ HTTP\/(.*)/i,"") if parts[0]
          if parts[0] and parts[0] =~ /.* (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)(.*\/)/i then          
            return $2
          else
            return "/"
          end
        rescue => bang
          puts "!!! HTTP-Parser (path): #{bang}"
        end
      end
      
      def element
        dummy = self.first        
        if dummy =~ /\w+ (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}).*\/(.*)\??.* HTTP/i then          
          return $2
        else
          return "/"
        end        
      end
      
      def doctype
        dummy = self.first.gsub(/\?+/,"?")
        parts = dummy.split('?')
        parts[0].gsub!(/ HTTP\/(.*)/i,"")
        if parts[0] =~ /(.*\.)(\w{2,3})$/i then          
          return $2
        else
          return ''
        end
      end
      
      def proto
        dummy = self.first
        result = "http"
        #  puts dummy
        if dummy =~ /\w+ https:\/\//i then          
          result = "https"
        end
        return result
      end
      
      def url        
        dummy = self.first.dup        
        dummy.gsub!(/ HTTP\/(.*)/i,"")
        dummy.gsub!(/^\w+ /,"")      
        return dummy.strip        
      end
      
      
      def site
        dummy = self.first.gsub(/\?+/,"?")
        parts = dummy.split('?')
        if parts[0] =~ /\w+ https?:\/\/([\-0-9a-zA-Z.]*[:0-9]{0,6})/i then          
          return $1
        else
          return nil
        end
      end
      
      def host
        dummy = self.first
        parts = dummy.split('?')
        if parts[0] =~ /\w+ https?:\/\/([\-0-9a-zA-Z.]*)[:0-9]{0,6}/i then          
          return $1
        else
          return nil
        end
      end
      
      def port
        dummy = self.first
        portnum = nil
        parts = dummy.split('?')
        if parts[0] =~ /\w+ https:\/\// then
          portnum = 443
        elsif parts[0] =~ /\w+ http:\/\//
          portnum = 80
        end
        if parts[0] =~ /\w+ https?:\/\/[\-0-9a-zA-Z.]*:([0-9]{0,6})/i then          
          portnum = $1
        end
        return portnum
      end
      
      def get_parms
        parmlist=[]
        if self.first =~ /\w+ (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}).*\/.*(\?.*=.*) HTTP/i then
          dummy = $2.gsub!(/\?+/,"?").split('?')
          # remove left part of ? from url 
          dummy.shift
          
          parmlist=dummy.join.split(/\&/)
        end
        return parmlist
        
      end
      
      def post_parms
        parmlist=[]
        if self.last =~ /\=.*\&?/i
          parmlist = self.last.split(/\&/)        
        end
        return parmlist
      end
      
      def parms
        parmlist=[]
        parmlist.concat(get_parms)
        parmlist.concat(post_parms)
        
        return parmlist
      end
      
      def parm_names
        
        parm_names=[]
        parmlist=[]
        parmlist.concat(get_parms)
        parmlist.concat(post_parms)
        parmlist.each do |p| 
          p.gsub!(/=.*/,'')
          parm_names.push p
        end
        
        return parm_names
        
      end
      
      def post_parm_names
        
        parm_names=[]
        parmlist=[]
        
        parmlist.concat(post_parms)
        parmlist.each do |p|
          if p then
            p.gsub!(/=.*/,'') 
            parm_names.push p
          end
        end
        
        return parm_names
        
      end
      
      def get_parm_names
        
        parm_names=[]
        parmlist=[]
        parmlist.concat(get_parms)
        
        parmlist.each do |p| 
          if p then
            p.gsub!(/=.*/,'')
            parm_names.push p
          end
        end
        
        return parm_names
        
      end
      
      def get_parm_value(parm_name)
        parm_value = ""
        self.get_parms.each do |parm|
          if parm =~ /^#{Regexp.quote(parm_name)}=/i then
            dummy = parm.split(/=/)
            if dummy.length > 1 then
            #  parm_value=dummy[1].gsub(/^[ ]*/,"")  
            parm_value=dummy[1].strip
            end
          end
        end
        return parm_value
      end
      
      def post_parm_value(parm_name)
        parm_value=""
        self.post_parms.each do |parm|
          if parm =~ /#{Regexp.quote(parm_name)}/i then
            dummy = parm.split(/=/)
            if dummy.length > 1 then
              parm_value = dummy[1].strip
            else
              # puts "Uhhhh ... need parameter value from '#{parm}''"
            end
          end
        end
        return parm_value
      end
      
      
      def headers
        header_list=[]
        self.each do |line|
          return header_list if line.strip.empty?
          header_list.push line
        end
        return header_list
      end
      
      def cookies
        cookie_list=[]
        self.headers.each do |line|
          if line =~ /Cookie2?: (.*)/ then
            clist = $1.split(";")
            clist.each do |c|
              c.gsub!(/^[ ]+/,"")
              c.chomp!
              cookie_list.push c
            end
          end
        end
        return cookie_list
      end
      
      def body
        body=[]
        eoh = false
        self.each do |line| 
          if not eoh and line.strip.empty? then
            eoh = true
            next
          end          
          if eoh == true then body.push line;end
        end
        return body
        
      end
      
      
      
      def header_value(header_name)
        header_values =[]
        self.headers.each do |header|
          if header =~ /^#{header_name}/i then
            dummy = header.split(/:/)
            value=dummy[1]
            value.gsub!(/^[ ]*/,"")
            header_values.push value
          end
        end
        return header_values
      end
      
      def content_type
        ct = "undefined"
        self.each do |line|
          break if line.strip.empty?
          if line =~ /^Content-Type: (.*)/ then 
            ct = $1
            break
          end
        end
        return ct
      end
      
      def content_length
        ct = 0
        self.each do |line|
          break if line.strip.empty?
          if line =~ /^Content-Length: (.*)/i then 
            ct = $1.to_i
            break
          end
        end
        return ct
      end
      
      def transferEncoding
        te = TE_NONE
        self.each do |line|
          break if line.strip.empty?
          if line =~ /^Transfer-Encoding: (.*)/i then
            dummy = $1.strip
            puts dummy
            te = case dummy 
              when 'chunked'
                Watobo::Core::TE_CHUNKED
              when 'compress'
                Watobo::Core::TE_COMPRESS
              when 'zip'
                Watobo::Core::TE_GZIP
              when 'deflate'
                Watobo::Core::TE_DEFLATE
              when 'identity'
                Watobo::Core::TE_IDENTITY
              end
            break
          end
        end
        return te
      end
      
      def status
        dummy = self.first
        
        if dummy =~ /^HTTP\/1.. (.*)/i then          
          return $1.chomp
        else
          return nil
        end        
      end
      
      def contentMD5
        hash = Digest::MD5.hexdigest(self.body.join)
        return hash
      end
    end
    
    ############################################################
    # R E S P O N S E   P A R S E R   M I X I N 
    ###############################################################
    module ResponseParser
      def status
        dummy = self.first
        
        if dummy =~ /HTTP\/1.. (.*)/i then          
          return $1.chomp
        else
          return nil
        end        
      end
      
      
      
      def headers
        header=[]
        self.each do |line|
          return header if line.length <3 or line =~ /^\r\n$/
          header.push line
        end
      end
      
      
    end
  end
end

