# .
# interceptor.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/httpparser.rb'
require 'lib/qRequest.rb'
require 'lib/qShapers.rb'
require 'lib/guiUtils.rb'
require 'lib/qProject.rb'
require 'lib/http_socket.rb'
require 'lib/core/watobo_constants'
require 'socket'

require 'fox16'
require 'pathname'

include Fox
#
#   I N T E R C E P T P R O X Y
#
class InterceptProxy
  attr_accessor :interactive
  
  def addPreview(hashid, response)
    @preview[hashid] = response  
  end
  
  def stop()
    @intercept_srv.close
  end
  
  def read_request(socket)
    request = []
    # read http header lines
    session = socket
    # puts "* read header ..."
    Watobo::Framework::HTTP.read_header(socket) do |line|
      request.push line
    end
    
    # puts "* got header (#{request.join.length})"
    
    if request.first =~ /^CONNECT/ then
      is_ssl = true
      
      if request.first =~  /^(\w*) (.*):(\d{1,4})/ then
        method = $1
        target = $2
        tport = $3
        # puts request.first
        print "\n* CONNECT: #{method} #{target} on port #{tport}\n"
      else
        puts "* could not parse target port"
        puts "-"
        puts request.first
        puts "-"
        return
      end
      socket.print "HTTP/1.0 200 Connection established\r\n" +
         "Proxy-connection: Keep-alive\r\n" +
                   "Proxy-agent: WATOBO-Proxy/1.1\r\n" +
                   "\r\n"
      bscount = 0 # bad handshake counter
      puts "* wait for ssl handshake ..."
      begin
        ssl_socket = OpenSSL::SSL::SSLSocket.new(socket, @ctx)
        ssl_socket.setsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1)
        #ssl_socket.sync_close = true
        ssl_socket.sync = true
        # puts ssl_socket.methods.sort
        
        ssl_session = ssl_socket.accept 
        
        
      rescue => bang
        
        #puts ssl_session
        if not ssl_session then bscount += 1;end
        if bscount > 5 then
          puts "!!! Error: SSL-Handshake with Client/Browser"
          puts bang
          return nil, socket
        end
        retry
      end
      session = ssl_session
      puts "* ssl ok!"
      # now read ssl request header
      request = []
      Watobo::Framework::HTTP.read_header(session) do |line|
        request.push line
      end
      return request, session if not request.first
      request.first.gsub!(/(\w+)( )(\/.*)/, "\\1 https://#{target}:#{tport}\\3")
    end
    #puts request
    request.extend Watobo::Parser::Web10
    clen = request.content_length
    if  clen > 0 then
      body = ""
      Watobo::Framework::HTTP.read_body(session) do |data|
        body += data
        break if body.length == clen
      end
      request.push body
    end
    
    
    return request, session
  end
  
  #
  # R U N
  #
  
  def run
    begin
      
      @intercept_srv = TCPServer.new("127.0.0.1", @port)
      @intercept_srv.setsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1)
    rescue => bang
      puts "\n!!!Could not start InterceptProxy"
      puts bang
      return nil
    end
    puts "\n* Interception started on port #{@port}"
    session_list = []
    Thread.new(@intercept_srv) { |server|
      while (new_session = server.accept)
        Thread.new(new_session) { |session|
          begin 
            request, session_new = read_request(session)
            if not request then
              session_new.close
              next
              #return
            end
            puts request.first
            puts "-"
          rescue => bang
            puts "!!! Error reading client request "
            puts bang
            next
            #break
          end
          
          # puts request
          if request.first =~ /WATOBOPreview=([0-9a-zA-Z]*)/ then
            puts "* preview requested ..."
            puts request.first
            
            hashid = $1
            response = @preview[hashid] 
            
            if response then
             # puts response
              session_new.write response.join
              if session_new.class.to_s =~ /SSLSocket/
                session_new.io.shutdown(2)
              else
                session_new.shutdown(2)
              end
              session_new.close
            end
          else
            if @project.intercept_request then
              if @project.interceptor then
                
                @project.interceptor.modifyRequest(request, Thread.current)  
                Thread.stop
              end
            end
            
            begin
              req, resp = @sender.sendRequest(request, :update_sids => false, :update_session => false, :update_contentlength => true)
            rescue => bang
              puts "* Error sending request"
            end
            #
            if resp then 
              begin
                if @project.intercept_response then
                  
                  if resp.content_type =~ /text/ or resp.content_type =~ /application\/javascript/ then
                    @project.interceptor.modifyResponse(resp, Thread.current)  
                    Thread.stop
                    
                  end
                  
                end
                
                session_new.write resp.join
                if session_new.class.to_s =~ /SSLSocket/
                  session_new.io.shutdown(2)
                else
                  session_new.shutdown(2)
                end
                session_new.close
                
                if req then # when req == nil an error occured -> no chat necessary
                  chat = Qchat.new(request, resp, @project.last_chatid + 1)
                  @project.addChat(chat)
                end
              rescue => bang
                puts "!!! Error sending response to client"
                puts bang
                #  ensure
                #    session_new.close
              end
            else
              puts "* got no response"
              session_new.close
            end
          end
        }
        
      end
    }
  end
  
  def initialize(project, cert_path)
    begin
      @project = project
      @port = @project.settings[:intercept_port]
      @sender = InterceptSender.new(@project)
      @interactive = false
      @intercept_srv = nil
      @preview = Hash.new
      @preview['ProxyTest'] = ["HTTP/1.0 200 OK\r\nServer: Watobo-Interceptor\r\nConnection: close\r\nContent-Type: text/html; charset=iso-8859-1\r\n\r\n<html><body>PROXY_OK</body></html>"]
      
      cert_file = File.join(cert_path, "cert.pem")
      key_file = File.join(cert_path, "privkey.pem")
      
      #  @ctx = OpenSSL::SSL::SSLContext.new('SSLv23_server')
      @ctx = OpenSSL::SSL::SSLContext.new()
      
      @ctx.cert = OpenSSL::X509::Certificate.new(File.read(cert_file))
      #  @ctx.key = OpenSSL::PKey::DSA.new(File.read(key_file))
      @ctx.key = OpenSSL::PKey::RSA.new(File.read(key_file))
      @ctx.verify_mode = OpenSSL::SSL::VERIFY_NONE
      @ctx.timeout = 10
      #  @ctx.ciphers = nil # ['TLSv1/SSLv3', 56, 56 ]
      
      
    rescue => bang
      puts "!!!could not read certificate files:"
      puts cert_file
      puts key_file
      puts bang
    end
    
  end
end

class InterceptSender < Watobo::Framework::Session
  def initialize(project)
    super(project)
  end
  
  def runLogin(prefs)
    @project.loginscript_ids.each do |chatid|
      #     puts "login request #{chatid}"
      test_req = @project.getChat(chatid).copyRequest
      
      #  puts test_req
      
      request,response = doRequest(test_req,:update_sids => true, :update_session => true, :update_contentlength => true)
      puts 
      #   puts "+ Login-Request:"
      #  puts request
      #   puts
      #   puts "+ Login-Response:"
      #   puts response
    end
  end
  
  
  def sendRequest(request, prefs)
    
    if prefs[:run_login ] == true
      runLogin(prefs)
    end
    new_request = []
    request.each do |line|
      new_request.push line.clone
    end
    new_request.extend Watobo::Parser::Web10
    new_request.extend Watobo::Framework::Shaper::Web10
    
    test_req, test_resp = self.doRequest(new_request, prefs)
    
    return test_req,test_resp
  end
end

#
#--------------------------------------------------------------------------------------------
#

# class FXInterceptor < FXDialogBox
class FXInterceptor < FXTopWindow
  
  include Responder
  include Watobo::Core
   
  def execute
    create
    show(PLACEMENT_SCREEN)
    # getApp().runModalFor(self)
  end
  
  def compileNewResponse(response)
    resp = response.parseRequest()
    resp.extend Watobo::Parser::Web10
    resp.extend Watobo::Framework::Shaper::Web10
    resp.fixupContentLength()
    
    return resp
  end
  
  def onInterceptChanged(sender, sel, ptr)
    if not @project.nil? then
      @project.intercept_response = @intercept_response.checked? ? true : false
      @project.intercept_request = @intercept_request.checked? ? true : false
    end
  end
  
  def onAcceptChanges(sender, sel, ptr)
    if @tabBook.current == 0 then
      begin
        @edit_request[:request].clear
        @edit_request[:request].concat @requestbox.parseRequest
        
        @edit_request[:thread].run
        getNextRequest()
      rescue => bang
        puts "!!! Error"
        puts bang
      end
    else
      begin
        @edit_response[:response].clear
        new_response = compileNewResponse(@responsebox)
        @edit_response[:response].concat new_response
        
        @edit_response[:thread].run
        getNextResponse()
      rescue => bang
        puts "!!! Error"
        puts bang
      end
    end
    
  end
  
  def onDrop(sender, sel, ptr)
    if @tabBook.current == 0 then
      @edit_request[:request].clear
      
      @edit_request[:thread].run
      getNextRequest()
    else
      @edit_response[:response].clear
      
      @edit_response[:thread].run
      getNextResponse()
      
    end
  end
  
  def getNextResponse
    @edit_response = @response_list.shift
    if @edit_response then
      puts "* edit next request"
      @responsebox.setText(@edit_response[:response])
      @response_tab.text = "Response (#{@request_list.length+1})"
    else 
      disable_buttons()
      @responsebox.setText('')
      @response_tab.text = "Response (0)"
    end
  end
  
  def getNextRequest
    @edit_request = @request_list.shift
    if @edit_request then
      puts "* edit next request"
      @requestbox.setText(@edit_request[:request])
      @request_tab.text = "Request (#{@request_list.length+1})"
    else 
      disable_buttons()
      @requestbox.setText('')
      @request_tab.text = "Request (0)"
    end
  end
  
  def onDiscard(sender, sel, ptr)
    if @tabBook.current == 0 then
      @edit_request[:thread].run
      getNextRequest()
    else
      @edit_response[:thread].run
      getNextResponse()
    end
  end
  
  def onDiscardAll(sender, sel, ptr)
    
  end
  
  def onHide
    puts "* hiding interceptor"
    @project.intercept_response = false
    @project.intercept_request = false
    @project.interceptor = nil
    @mutex.synchronize {
      @cv.signal
    }
  end
  
  def releaseAll()
    puts "* closing interceptor"
    puts "* disable all interceptions"
    @project.intercept_request = false
    @project.intercept_response = false
    puts "* release all interceptions ..."
    @request_list.each do |ir|
      ir[:thread].run
    end
    @response_list.each do |ir|
      ir[:thread].run
    end
    
    
  end
  
  def onClose(sender, sel, ptr)
    releaseAll()    
    #getApp().stopModal(self, 1)
    self.hide()
  end
  
  
  def modifyRequest(request, thread)
    puts "* FXInterceptor: modifyRequest"
    new_request = {
      :request => request, 
      :thread => thread
    }
    if not @edit_request then
      @edit_request = new_request
      
      @requestbox.setText(request)
      @tabBook.setCurrent(0)
      @request_tab.text = "Request (1)"
    else
      @request_list.push new_request
      @request_tab.text = "Request (#{@request_list.length+1})"
    end
    
    enable_buttons()
  end
  
  def modifyResponse(response, thread)
    response.extend Watobo::Parser::Web10
    response.extend Watobo::Framework::Shaper::Web10
    response.fixupContentLength()
    
    new_response = {
      :response => response, 
      :thread => thread
    }
    if not @edit_response then
      @edit_response = new_response
      @responsebox.setText(response)
      @tabBook.setCurrent(1)
      enable_buttons() 
      @response_tab.text = "Response (1)"
    else
      @response_list.push new_response
      @response_tab.text = "Response (#{@request_list.length+1})"
    end    
    enable_buttons()
  end
  
  def enable_buttons
    @accept_button.enabled = true
    @discard_button.enabled = true
    @drop_button.enabled = true
  end
  
  def disable_buttons
    @accept_button.enabled = false
    @discard_button.enabled = false
    @drop_button.enabled = false   
  end
  
  
  
  def initialize(owner, project, opts)
    # Invoke base class initialize function first
      
    super( owner, 'Interceptor', nil, nil, DECOR_ALL|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE, 0, 0, 500, 500, 0, 0, 0, 0, 0, 0)
    self.connect(SEL_CLOSE, method(:onClose))
       
    @project = project
    @request_list = []
    @response_list = []
    @edit_request = nil
    @edit_response = nil
        
    # initial frame setup
    mr_splitter = FXSplitter.new(self, LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_VERTICAL|SPLITTER_REVERSED|SPLITTER_TRACKING)
    
    top_frame = FXVerticalFrame.new(mr_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y||LAYOUT_FIX_HEIGHT|LAYOUT_BOTTOM,:height => 500) 
    top_splitter = FXSplitter.new(top_frame, LAYOUT_FILL_X|SPLITTER_HORIZONTAL|LAYOUT_FILL_Y|SPLITTER_TRACKING)
    
    log_frame = FXVerticalFrame.new(mr_splitter, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM,:height => 100)
    
    filter_frame = FXVerticalFrame.new(top_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y||LAYOUT_FIX_HEIGHT|LAYOUT_BOTTOM)
    view_frame = FXVerticalFrame.new(top_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y||LAYOUT_FIX_HEIGHT|LAYOUT_BOTTOM)
    
    
    @tabBook = FXTabBook.new(view_frame, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
    button_frame = FXHorizontalFrame.new(view_frame, LAYOUT_FILL_X|LAYOUT_FILL_Y) 
    
    @request_tab = FXTabItem.new(@tabBook, "Request", nil)  
    request_frame_outer = FXVerticalFrame.new(@tabBook, LAYOUT_FILL_X|LAYOUT_FILL_Y)
    request_frame = FXVerticalFrame.new(request_frame_outer, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding=>0)
    @requestbox = Watobo::Gui::RequestEditor.new(request_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    @requestbox.editable = true
    
    
    @response_tab = FXTabItem.new(@tabBook, "Response", nil)
    response_frame_outer = FXVerticalFrame.new(@tabBook, LAYOUT_FILL_X|LAYOUT_FILL_Y)
    response_frame = FXVerticalFrame.new(response_frame_outer, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding=>0)
    @responsebox = Watobo::Gui::RequestEditor.new(response_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y )
    @responsebox.editable = true
    
    
    FXLabel.new(filter_frame, "Intercept:" )
    @intercept_request = FXCheckButton.new(filter_frame, "Requests", nil, 0,
    ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    @intercept_request.checkState = @project.intercept_request
    @intercept_request.connect(SEL_COMMAND, method(:onInterceptChanged))
    
    
    @intercept_response = FXCheckButton.new(filter_frame, "Response", nil, 0,
    ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    
    @intercept_response.checkState = @project.intercept_response
    @intercept_response.connect(SEL_COMMAND, method(:onInterceptChanged))
    
    @accept_button = FXButton.new(button_frame, "Accept", nil, nil, 0, FRAME_RAISED|FRAME_THICK|LAYOUT_LEFT)
    @accept_button.connect(SEL_COMMAND, method(:onAcceptChanges))
    
    @discard_button = FXButton.new(button_frame, "Discard", nil, nil, 0, FRAME_RAISED|FRAME_THICK|LAYOUT_LEFT)
    @discard_button.connect(SEL_COMMAND, method(:onDiscard))
    
    @drop_button = FXButton.new(button_frame, "Drop", nil, nil, 0, FRAME_RAISED|FRAME_THICK|LAYOUT_LEFT)
    @drop_button.connect(SEL_COMMAND, method(:onDrop))
    
    log_text_frame = FXVerticalFrame.new(log_frame, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding=>0)
    @log_viewer = FXText.new(log_text_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
    disable_buttons()   
    
  end
end

