# .
# qGui.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'fox16'
require 'lib/core/watobo_constants'
require 'lib/utils/save_settings'
require 'lib/utils/load_settings'
require 'lib/qClipboard.rb'
require 'lib/qLoginWizzard.rb'
require 'lib/qManualRequest.rb'
require 'lib/encoders.rb'
require 'lib/guiUtils.rb'
require 'lib/interceptor.rb'
require 'lib/transcoder.rb'
require 'lib/chatviewer.rb'
require 'lib/requesteditor.rb'
require 'lib/gui/project_wizzard.rb'
require 'lib/gui/workspace_dialog.rb'
require 'lib/gui/findings_tree.rb'
require 'lib/gui/sites_tree.rb'
require 'lib/gui/progress_window'
require 'lib/gui/proxy_dialog.rb'
require 'lib/gui/fuzzer_gui'
require 'lib/gui/check_policy_menu'
require 'lib/gui/dashboard'
require 'lib/gui/certificate_dialog'
require 'lib/gui/finding_info'
require 'lib/gui/start_full_scan_dialog'
require 'lib/gui/start_quick_scan_dialog'
require 'lib/gui/session_management_dialog'
require 'lib/gui/preferences_dialog'
require 'lib/gui/browser_preview'
require 'lib/gui/status_bar'
require 'lib/gui/about_watobo'
require 'lib/utils_include.rb'
require 'lib/core/active_check.rb'

include Fox

def getList(pattern)
  vals = pattern.split(",")
  result = []
  vals.each do |v|
    v.strip!
    if v =~ /^(\d+)$/ then
      result.push $1.to_i
    elsif v =~ /^(\d+)-(\d+)$/
      start = $1
      stop = $2
      dummy = (start..stop).to_a
      result.concat dummy 
    end
  end
  result.uniq!
  return result
end


#===================================================================================================================

####################################################################################################################
# M A I N   A P P L I C A T I O N   W I N D O W
#
class WatoboMainWindow < FXMainWindow
  include Watobo::Gui
  include Watobo::Gui::Utils
  
  attr :interceptor
  attr :default_settings
  attr :watobo_base
  attr :active_project
  attr_accessor :iproxy
  
  # Handles SEL_CLOSE from the main window
  def onClose(sender, sel, event)
    #  puts "! #{Thread.list.length} Threads running"
    response = FXMessageBox.question(self, MBOX_YES_NO, "Finished?", "Are you sure?")
    if response == MBOX_CLICKED_YES           
      return 0
    else
      return 1
    end    
  end
  
  def siteIsFiltered?(site)
    self.default_settings[:domain_filters].each do |domainfilter|
      if site =~ /#{domainfilter}/ then
        return true
      end
      #   puts site + " : " + domainfilter if site =~ /google/
    end
    
    false
  end
  
  def chatIsFiltered?(chat)
    begin
      
      if not @doctype_TableFilter.include?(chat.request.doctype) and not siteIsFiltered?(chat.request.site) then 
        if @table_filter == "" then
          return false
        elsif @foption_req.checked? and chat.request.join =~ /#{@table_filter}/i then
          return false
        elsif @foption_res.checked? and chat.response.join =~ /#{@table_filter}/i then
          return false
          
        elsif chat.request.first =~ /#{@table_filter}/i then
          return false
        end
      end
      return true
    rescue => bang
      puts "!!!ERROR: could not add chat to table:"
      puts chat.id
      puts bang
      return false
    end 
  end
  
  def saveDefaultSettings(new_settings) 
    if new_settings then
      
    
      @default_settings.update new_settings
      
    end
    
    Watobo::Utils.save_settings(@default_settings_file, 0, "default_settings", @default_settings)
  end
  
  def checkProxySettings()
    begin
      if not @browserView.watoboProxy? then
        puts "... settings [FALSE]"
        @browserViewEnabled = false
        FXMessageBox.information(self,MBOX_OK,"Proxy Settings", "Your InternetExplorer does not use WATOBO as its proxy.\nSo you can't use the Browser-View feature.\nPlease change your proxy settings and try it again!")
      else
        @browserViewEnabled = true
        puts "... settings [OK]"
      end
    rescue => bang
      puts "!!!ERROR: Could not check proxy settings"
      puts bang
    end
  end
  
  
  def updateRequestTable(project)
    uniqueHashes = Hash.new
    # first initialize the table 
    @chatTable.clearConversation()
    project.chat_list.each do |chat|
      if chatIsFiltered?(chat) == false 
        
        uh = @active_project.uniqueRequestHash(chat.request)
        if @table_option_unique.checked? and not uniqueHashes.has_key?(uh)
          @chatTable.addChat(chat)
          uniqueHashes[uh] = chat
        elsif not @table_option_unique.checked?
          @chatTable.addChat(chat)
        else
          puts "* skipping chat[#{chat.id}] - hash(#{uh}) already exists"
        end
        
      end
    end
  end
  
  
  #
  # SHOW CHAT
  #
  def show_chat(chat)    
    @request_viewer.setText(chat.request)  
    @last_request = chat.request
    @response_viewer.setText(chat.response)
    @last_response = chat.response
    @switcher.current=0
    @lastViewed = chat
    
    @viewContent.text = "Chat-ID: #{chat.id}"
  end
  
  #
  # SHOW VULN
  #
  def show_vuln(vuln)
    @chatTable.killSelection()
    @request_viewer.setText(vuln.request)
    @last_request = vuln.request
    @response_viewer.setText(vuln.response)
    @last_response = vuln.response
    
    @lastViewed = vuln
    if vuln.info[:check_pattern] then
      
      pattern = vuln.info[:check_pattern].strip
      
      @request_viewer.highlight(pattern)
    end
    
    if vuln.info[:proof_pattern] then
      pattern = vuln.info[:proof_pattern].strip
      @response_viewer.highlight(pattern)
    end
    @switcher.current = 0
    
    @viewContent.text = "Finding \"#{vuln.info[:class]}\" [Chat-ID: #{vuln.info[:chat].id}]"
    
  end
  
  
  def addFinding(finding)
    @findings_tree.addFinding(finding)
  end
  
  def openSessionManagement(sender, sel, item)
    smdlg = SessionManagementDialog.new(self, @active_project)
    if smdlg.execute != 0 then
      ids = smdlg.getLoginScriptIds()
      sidpatterns = smdlg.getSidPatterns()
      logout_signatures = smdlg.getLogoutSignatures()
      if @active_project
        @active_project.loginscript_ids = ids 
        @active_project.settings[:sid_patterns] = sidpatterns
        @active_project.settings[:logout_signatures] = logout_signatures
      end
      # save settings
      @active_project.saveProjectSettings()
      @active_project.saveSessionSettings()
    end
  end
  
  def openPreferencesDialog(sender, sel, ptr)
    prefdlg = PreferencesDialog.new(self, @default_settings)
    if prefdlg.execute != 0
      
      if @default_settings[:intercept_port] != prefdlg.settings[:intercept_port] then
        FXMessageBox.information(self,MBOX_OK,"Intercept Port Changed", "Please restart WATOBO!")
      end
      @default_settings.update(prefdlg.settings)
      puts prefdlg.settings
      
      saveDefaultSettings(nil)
      if @active_project then
        @active_project.settings.update(@default_settings)
        @active_project.saveProjectSettings()
      end
      
    end
  end
  
  def openCADialog(sender, sel, item)
    cadlg = CertificateDialog.new(self, @active_project)
    if cadlg.execute != 0
      
    end
  end
  
  def open_manual_request_toolkit(chat) 
    begin
      mrtk = QXManualRequest.new(FXApp.instance, @active_project, chat)
      mrtk.create
      mrtk.show(Fox::PLACEMENT_SCREEN)
    rescue => bang
      puts "!!! could not open manual request"
      puts bang
    end
  end
  
  def openFuzzer(chat) 
    begin
      fuzzer = FuzzerGui.new(FXApp.instance, @active_project, chat)
      fuzzer.create
      fuzzer.show(Fox::PLACEMENT_SCREEN)
    rescue => bang
      puts "!!! could not open manual request"
      puts bang
    end
  end
  
  
  def addChat(session, chat)
    # addChatToTable(chat) if chatIsFiltered?(chat) == false
    @chatTable.addChat(chat) if chatIsFiltered?(chat) == false     
    @sites_tree.addChat(chat)
    
  end
  
  def showPassiveCheckStatus
    @switcher.current=2
    @dashboard.tabBook.current = 1
  end
  
  def showActiveCheckStatus
    @switcher.current=2
    @dashboard.tabBook.current = 2
  end
  
  def showDashboard(sender, sel, ptr)
    #@switcher.current=2
    begin
      @switcher.setCurrent(2, true)
    rescue
      puts "no dashboard available yet!"
    end
  end
  
  def showLogs(sender, sel, ptr)    
    @switcher.current=1    
  end
  
  def showConversation(sender, sel, item)    
    @switcher.current=0    
  end
  
  
  def openBrowser(request, response)
    begin
      if @browserViewEnabled then
        @browserView.show(@iproxy, request, response) 
      else
        checkProxySettings()
        @browserView.show(@iproxy, request, response) if @browserViewEnabled
      end
    rescue => bang
      puts "!!!ERROR: Could not show preview"
      puts bang
    end
  end
  
  def updateTreeLists()
    @findings_tree.reload()
    @sites_tree.reload()
  end
  
  def openTranscoder(text2transcode)
    transcoder = FXTranscoder.new(FXApp.instance, text2transcode)
    transcoder.create
    transcoder.show(Fox::PLACEMENT_SCREEN)
  end
  
  def showFindingInfo(finding)
    @finding_info.showInfo(finding)
    @switcher.current = 3
  end
  
  def onOpenTranscoder(sender, sel, item)
    
    openTranscoder(nil)
    
    
  end
  
  def onExit(sender, sel, item)
    response = FXMessageBox.question(self, MBOX_YES_NO, "Finished?", "Are you sure?")
    if response == MBOX_CLICKED_YES           
      getApp().exit(0)
    end    
  end
  
  def onOpenInterceptor(sender, sel, ptr)
    if @active_project then
      interceptor = FXInterceptor.new(self, @active_project, :opts => DECOR_ALL)
      puts "* Interceptor created"
      @active_project.interceptor = interceptor
      interceptor.create
      interceptor.show(Fox::PLACEMENT_SCREEN)
      getApp().runModalWhileShown(interceptor)
      interceptor.releaseAll()
      puts "* Interceptor closed"
      #if interceptor.execute != 0 then
      #  puts "interceptor finished"
      #end
    else
      
      FXMessageBox.information(self,MBOX_OK,"No Project Defined", "Create Project First")
    end
  end
  
  def setupProgressWindow(title, numTotal)
    @progressWindow = Watobo::Gui::ProgressWindow.new(self, title, numTotal)
    
    @progressWindow.create
    @progressWindow.show(PLACEMENT_SCREEN)
    #  getApp().runModalWhileShown(@progressWindow)
    return @progressWindow
    
  end
  
  
  def closeProgressWindow
    # getApp().stopModal()
    @progressWindow.finished
  end
  
  def disableFileMenu
    @file_new_menu.enabled = false
    @file_history.disable
    @new_project_button.disable
  end
  
  def closeProject()
    @active_project = nil
    @findings_tree.clearItems()
    @sites_tree.clearItems()
    @chatTable.clearItems()
    @request_viewer.setText('')
    @response_viewer.setText('')
    @lastViewed = nil
    @last_request = nil
    @last_response = nil
    @iproxy.stop if @iproxy
  end
  #
  # onNewProject
  #
  def onNewProject(sender,sel,ptr)
    
    if @active_project then
      response = FXMessageBox.question(self, MBOX_YES_NO, "New Project", "This will close the actual project!\nAre you sure?")
      return 0 if not response == MBOX_CLICKED_YES           
      # clear old project
      closeProject()
      # stop interceptor   
    end
    
    newProjectWizzard = Watobo::Gui::NewProjectWizzard.new(self, @default_settings)
    if newProjectWizzard.execute != 0
      # prepare project settings
      project_settings = {
        :project_path => newProjectWizzard.selected_project_path,
        :session_path => newProjectWizzard.selected_session_path,
        :project_name => newProjectWizzard.project_name,
        :session_name => newProjectWizzard.session_name,
        :module_path => File.join(@watobo_base, "modules"),
        :intercept_port => @default_settings[:intercept_port],
        :proxy_list => @default_settings[:proxy_list],
        :proxy => "",
        :domain_filters => @default_settings[:domain_filters],
        :sid_patterns => @default_settings[:sid_patterns],
        :logout_signatures => @default_settings[:logout_signatures],
        :active_policy => @default_settings[:active_policy],
        :active_policies => @default_settings[:active_policies],
      }
      # set workspace_dir
      @default_settings[:workspace_dir] = newProjectWizzard.workspace_dir
      # setup project settings filename -> history
      file = project_settings[:session_name] + ".wss"
      session_settings_file = File.join(project_settings[:session_path], file) 
      @default_settings[:history].unshift  session_settings_file if not @default_settings[:history].include?(session_settings_file)
      @default_settings[:history].pop if @default_settings[:history].length > 5
      
      # save default settings
      saveDefaultSettings(nil)
      
      puts "creating new project ..."
      print "Name: "
      print "#{project_settings[:project_name]}\n\n"
      
      startProject(project_settings)
      
    end
    
  end
  
  def startProject(project_settings)
    progressWindow = Watobo::Gui::ProgressWindow.new(self, "New Project", 0)
    project_settings[:progress_window] = progressWindow
    
    progressWindow.create
    progressWindow.show(PLACEMENT_SCREEN)
    progressWindow.newProject(@active_project, project_settings)
    
    getApp().runModalWhileShown(progressWindow)
    
    if not @active_project then
      puts "!!!Error: Could not create project" 
    end
    #create project
    #    t = Thread.new(project_settings) { |settings|
    begin
      
      #  @active_project = Qproject.new(self, settings)
      
      # puts "new active project"
      @project_list.push @active_project
      @findings_tree.project = @active_project
      @sites_tree.project = @active_project
      
      updateRequestTable(@active_project)
      
      @statusBar.projectName = @active_project.settings[:project_name]
      @statusBar.sessionName = @active_project.settings[:session_name]
      @statusBar.portNumber = @active_project.settings[:intercept_port].to_s
      @statusBar.statusInfo = "Ready"
      @start_scan_button.enable
      @dashboard.updateProjectInfo(@active_project)
    rescue => bang
      puts "!!! Could not create project"
      puts bang
    end
    #    }  
    #  t.join
    
  end
  
  def openSession(session_file)
    
    if @active_project then
      response = FXMessageBox.question(self, MBOX_YES_NO, "New Project", "This will close the actual project!\nAre you sure?")
      return 0 if not response == MBOX_CLICKED_YES           
      # clear old project
      closeProject()
      # stop interceptor   
    end
    if File.exists?(session_file) then
      session_settings = Watobo::Utils.loadSettings(session_file)
    else
      puts "!!! Session file does not exist (#{session_file})."
      return nil
    end
    if session_file =~ /(.*\/#{session_settings[:project_name]})\/#{session_settings[:session_name]}\/#{session_settings[:session_name]}/ then
      project_path = $1
    end
    if project_path then
      project_file = File.join( project_path, session_settings[:project_name]+".wps")
    else
      puts "! Wrong Session File (#{session_file})"
    end
    
    session_settings[:project_path] = project_path
    session_settings[:session_path] = File.join(project_path, session_settings[:session_name])
    
    if File.exists?(project_file) then
      project_settings = Watobo::Utils.loadSettings(project_file)
      project_settings.update(session_settings)
      startProject(project_settings)
    else
      puts "!!! No project file available (#{project_file})."
      return nil
    end
    
    return 1
  end
  
  def setActiveProject(project)
    @active_project = project
  end
  
  
  def startFullScan(sender,sel,ptr)
    dlg = Watobo::Gui::FullScanDialog.new(self, @active_project, LAYOUT_FILL_X|LAYOUT_FILL_Y)
    if dlg.execute != 0 then
      @start_scan_button.disable
      @pause_scan_button.enable
      @active_project.saveProjectSettings()
      @active_project.saveSessionSettings()
      
      @scanner = @active_project.createScanner()
      
      @dashboard.setupScanProgressFrame(@active_project.active_modules)
      
      Thread.new {
        begin
          @dashboard.setScanStatus("Running")
          @statusBar.statusInfo = "Full Scan Running"
          @scanner.run(:progress_displays => @dashboard.progressDisplays())
          logger("scan finished")
          @dashboard.setScanStatus("Finished")
          @statusBar.statusInfo = "Ready."
          @start_scan_button.enable
          @pause_scan_button.disable
        rescue => bang
          puts bang
        end
      }
    end
    
  end
  
  def pauseScan(sender, sel, ptr)
    begin
      if @scanner.running?
        @scanner.stop 
        @dashboard.setScanStatus("Scan Paused")
        @statusBar.statusInfo = "Scan Paused"
      else
        @scanner.continue
        @dashboard.setScanStatus("Scan Running")
        @statusBar.statusInfo = "Full Scan Running"
      end
    rescue => bang
      puts "!!!ERROR: Could not pause scanner"
      puts bang
    end
  end
  
  
  
  def applyFilter
    if @active_project then
      #  @doctype_TableFilter=""
      @doctype_TableFilter.clear
      if @foption_nopix.checked? then @doctype_TableFilter.concat(@fext_pix);end
      if @foption_nodocs.checked? then @doctype_TableFilter.concat(@fext_docs);end
      if @foption_nojs.checked? then @doctype_TableFilter.concat(@fext_javascript);end
      if @foption_nocss.checked? then @doctype_TableFilter.concat(@fext_style);end 
      
      updateRequestTable(@active_project)
    end
  end
  
  def onApplyFilterClick(sender,sel,item)
    applyFilter()  
    
  end
  
  def onClear(sender, sel, item)
    @table_filter.value =""
    @tableFilterFX.handle(self, FXSEL(SEL_UPDATE, 0), nil)
  end
  
  
  def onTableClick(sender,sel,item)
    begin
      getApp().beginWaitCursor()
      # purge viewers 
      @request_viewer.setText('')
      @response_viewer.setText('')
      row = item.row
      
      chatid = @chatTable.getRowText(row).to_i
      @chatTable.selectRow(row, false)
      # @logText.appendText("selected ID: (#{chatid})\n")
      @active_project.chat_list.each do |chat|
        if chat.id == chatid then
          show_chat(chat)
        end
      end
    rescue => bang
      puts "!!!ERROR: onTableClick"
      puts bang
      puts "!!!"
    ensure
      getApp().endWaitCursor()
    end
  end
  
  def onMenuProxy(sender,sel,item)
    if @active_project
      proxy_dialog = Watobo::Gui::ProxyDialog.new(self, @active_project.settings[:proxy], @active_project.settings[:proxy_list])
      if proxy_dialog.execute != 0 then
        @active_project.settings[:proxy_list] = proxy_dialog.getProxyList
        @active_project.settings[:proxy] = proxy_dialog.proxy
        puts "+ set proxy: #{@active_project.settings[:proxy]}"
        saveDefaultSettings(:proxy_list => @active_project.settings[:proxy_list])
        @active_project.saveProjectSettings()
      end
    else
      FXMessageBox.information(self,MBOX_OK,"No Project Defined", "Create Project First")
    end
  end
  
  def onCmdShowPreferences(sender, sel, ptr)
    prefDialog = FXpreferences.new(self)
    #  prefDialog.prefs[:source_path].value = @main.prefs[:source_path]
    #  prefDialog.prefs[:source_type].value  = @main.prefs[:source_type]
    
    @main.prefs.each do |key, value|
      if prefDialog.prefs[key]
        prefDialog.prefs[key].value = @main.prefs[key]
      end
    end
    
    if prefDialog.execute != 0
      
      @logText.appendText("Preferences changed...\n")
      prefDialog.prefs.each do |key, value|
        @main.prefs[key]=prefDialog.prefs[key].value
        
      end
      
    end
  end
  #
  # new_event
  #
  def logger(message)
    t = Time.now  
    now = t.strftime("%m/%d/%Y @ %H:%M:%S")  
    @logText.appendText("#{now}: #{message}\n")
    
  end
  
  def log(message)
    @logText.appendText("#{message}\n")
  end
  
  
  def init_main(qmain)
    @main=qmain
  end
  
  def initialize(app, prefs)
    # Invoke base class initialize first
    super(app, "WATOBO by siberas", :opts => DECOR_ALL, :width => 1000, :height => 600)
    # FXToolTip.new(app)
    
    self.icon = ICON_WATOBO
    self.show(PLACEMENT_MAXIMIZED)
    
    self.connect(SEL_CLOSE, method(:onClose))
    
    @watobo_base = prefs[:base_dir]
    
    @default_settings_file = File.join(@watobo_base, "watobo.wds")
    @saved_settings = nil
    
    
    
    @scanner = nil
    @iproxy = nil
    @browserView = BrowserPreview.new(nil)
    @browserViewEnabled = false
    
    @lastViewed = nil # last viewed/selected item (chat/finding)
    @last_request = nil
    @last_response = nil
    
    @default_settings = {
      :workspace_dir => "",
      :sid_patterns =>  [
       "name=\"(sessid)\" value=\"([0-9a-zA-Z!-]*)\"",
       "(sessid)=([-0-9a-zA-Z_:]*)(;|&)?",
       '(SESSIONID)=([-0-9a-zA-Z_:\.\(\)]*)(;|&)?',
       "(PHPSESSID)=([0-9a-zA-Z]*)(;|&)?",
       '(ASPSESSIONID)\w*=([0-9a-zA-Z]*)(;|&)?',
       "(MYSAPSSO2)=([0-9a-zA-Z.=%]*)(;|&)?", 
       "(ELEXIRSID)=([0-9a-zA-Z!-]*)(;|&)?",
       "(SLSID)=([0-9a-zA-Z!-]*)(;|&)?",
       "(sid)=([0-9a-z]*)(')?", #servlet?sid=9912ad967cc578a12ada85d91f841e18')
       '(saplb_\*)=([-0-9a-zA-Z_:\(\)]*)(;|&)?',
       "(DomAuthSessId)=([0-9a-zA-Z]*)(;|&)?",
       '(wgate)\/([\w]{4,}\/[\w=~]*)(;|&|\'|")?', # SAP ITS WGATE Session Handling
       '(session)=([-0-9a-zA-Z_:\.]*)(;|&)?'       # SAP ITS Session Cookie
      ],
      :logout_signatures => [
       "^Location.*login"
      ],
      :history => [],
      :intercept_port => 8081,
      :max_parallel_checks => 15,
      :proxy_list => {
        'webscarab' => "127.0.0.1:8008",
        'paros' => "127.0.0.1:8080",
      },
      :domain_filters => [
        '.*google.com',
        '.*tagesschau.de',
    '.*fxfeeds.mozilla.com',
    '.*ivwbox.de',
    '.*googlesyndication.com',
      
      ],
      :enable_smart_scan => true,
      :active_policy => 'Default',
      :active_policies => {
      'Default' => {
      'Watobo::Modules::Active::Directories::Dir_indexing' => false,
'Watobo::Modules::Active::Discovery::Http_methods' => false,
'Watobo::Modules::Active::Domino::Domino_db' => false,
'Watobo::Modules::Active::Sap::Its_commands' => false,
'Watobo::Modules::Active::Sap::Its_services' => false,
'Watobo::Modules::Active::Sap::Its_service_parameter' => false,
'Watobo::Modules::Active::Sap::Its_xss' => false,
'Watobo::Modules::Active::Sqlinjection::Sqli_simple' => true,
'Watobo::Modules::Active::Sqlinjection::Sql_boolean' => true,
'Watobo::Modules::Active::Sqlinjection::Sql_numerical' => false,
'Watobo::Modules::Active::Xss::Xss_simple' => true,
        },
      'All' => {
      'Watobo::Modules::Active::Directories::Dir_indexing' => true,
'Watobo::Modules::Active::Discovery::Http_methods' => true,
'Watobo::Modules::Active::Domino::Domino_db' => true,
'Watobo::Modules::Active::Sap::Its_commands' => true,
'Watobo::Modules::Active::Sap::Its_services' => true,
'Watobo::Modules::Active::Sap::Its_service_parameter' => true,
'Watobo::Modules::Active::Sap::Its_xss' => true,
'Watobo::Modules::Active::Sqlinjection::Sqli_simple' => true,
'Watobo::Modules::Active::Sqlinjection::Sql_boolean' => true,
'Watobo::Modules::Active::Sqlinjection::Sql_numerical' => true,
'Watobo::Modules::Active::Xss::Xss_simple' => true,
        }
        
      }
      
      
      
    }
    
    puts "* loading default settings from file #{@default_settings_file}"
    if File.exists?(@default_settings_file) then
      @saved_settings = Watobo::Utils.loadSettings(@default_settings_file)      
      @default_settings.update(@saved_settings)
    end
    
    @app = app
    @project_list = []
    
    @progressWindow = nil             # reserved for simple progress Window
    
    @switcher = nil
    
    @clipboard = Watobo::Framework::Clipboard.instance
    
    @interceptor = nil
    
    
    @doctype_TableFilter = []
    #--------------------------------
    # file extensions
    #--------------------------------
    @fext_pix=%w[ jpg gif bmp ico png ]
    @fext_docs=%w[ ppt doc xls pptx docx xlsx pdf ]
    @fext_javascript=%w[ js ]
    @fext_style=%w[ css ]
    
    @table_filter = FXDataTarget.new("")
    @active_project = nil
    
    
    
    menu_bar = FXMenuBar.new(self, :opts => LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
    
    file_menu_pane = FXMenuPane.new(self)
    FXMenuTitle.new(menu_bar, "File" , :popupMenu => file_menu_pane)
    @file_new_menu = FXMenuCommand.new(file_menu_pane, "New/Open" )
    @file_new_menu.connect(SEL_COMMAND, method(:onNewProject))
    
    exit_menu = FXMenuCommand.new(file_menu_pane, "Exit" )
    #FXMenuCommand.new(file_menu_pane, "Exit", nil, getApp(), FXApp::ID_QUIT)
    exit_menu.connect(SEL_COMMAND, method(:onExit))
    
    FXMenuSeparator.new(file_menu_pane)
    
    
    
    submenu = FXMenuPane.new(self) do |session_menu|
      
      @default_settings[:history].each do |h|
        if File.exists?(h)
          history = FXMenuCommand.new(session_menu, h.sub(@default_settings[:workspace_dir],'') )
          history.connect(SEL_COMMAND) do |sender, sel, item|
            puts "open session #{h}"
            openSession(h)
            
          end
        end
      end
    end
    @file_history = FXMenuCascade.new(file_menu_pane, "Recent Sessions", nil, submenu)
    
    #  file_open_command = FXMenuCommand.new(file_menu_pane, "Open..." )
    #  file_save_command = FXMenuCommand.new(file_menu_pane, "Save" )
    #  file_save_as_command = FXMenuCommand.new(file_menu_pane, "Save As..." )
    
    
    settings_menu_pane = FXMenuPane.new(self)
    FXMenuTitle.new(menu_bar, "Settings" , :popupMenu => settings_menu_pane)
    menu_proxy = FXMenuCommand.new(settings_menu_pane, "Forwarding Proxy..." )
    menu_session = FXMenuCommand.new(settings_menu_pane, "Session Management..." )
    
    menu_ca = FXMenuCommand.new(settings_menu_pane, "Create Certificate..." )
    menu_ca.connect(SEL_COMMAND, method(:openCADialog))
    
    menu_preferences = FXMenuCommand.new(settings_menu_pane, "Preferences..." )
    menu_preferences.connect(SEL_COMMAND, method(:openPreferencesDialog))
    # intercept_enable = FXMenuCheck.new(settings_menu_pane, "Enable Interception")
    
    # file_menu_title = FXMenuTitle.new(menu_bar, "Settings" , :popupMenu => settings_menu_pane)
    
    menu_proxy.connect(SEL_COMMAND, method(:onMenuProxy))
    menu_session.connect(SEL_COMMAND, method(:openSessionManagement))
    
    tools_menu_pane = FXMenuPane.new(self)
    FXMenuTitle.new(menu_bar, "Tools" , :popupMenu => tools_menu_pane)
    transcoder = FXMenuCommand.new(tools_menu_pane, "Transcoder")
    interceptor = FXMenuCommand.new(tools_menu_pane, "Interceptor")
    transcoder.connect(SEL_COMMAND, method(:onOpenTranscoder))
    interceptor.connect(SEL_COMMAND, method(:onOpenInterceptor))
    
    
    view_menu_pane = FXMenuPane.new(self)
    FXMenuTitle.new(menu_bar, "View" , :popupMenu => view_menu_pane)
    view_logs_command = FXMenuCommand.new(view_menu_pane, "Logs" )
    view_dashboard_command = FXMenuCommand.new(view_menu_pane, "Dashboard" )
    view_findings_command = FXMenuCommand.new(view_menu_pane, "Chat-Table")
    
    view_dashboard_command.connect(SEL_COMMAND, method(:showDashboard))
    view_logs_command.connect(SEL_COMMAND, method(:showLogs))
    view_findings_command.connect(SEL_COMMAND, method(:showConversation))
    
    help_menu_pane = FXMenuPane.new(self)
    FXMenuTitle.new(menu_bar, "Help" , :popupMenu => help_menu_pane)
    #   menu_lic = FXMenuCommand.new(help_menu_pane, "License" )
    menu_about = FXMenuCommand.new(help_menu_pane, "About" )
    menu_about.connect(SEL_COMMAND) {
      #FXMessageBox.information(self,MBOX_OK,"About", "WATOBO Version 0.9.1!")
      aboutDlg = AboutWatobo.new(self)
      aboutDlg.create
      aboutDlg.show(Fox::PLACEMENT_SCREEN)
    }
    tool_bar_shell = FXToolBarShell.new(self)
    top_dock_site = FXDockSite.new(self, LAYOUT_FILL_X|LAYOUT_SIDE_TOP)
    # bottom_dock_site = FXDockSite.new(self, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM)
    
    
    project_bar = FXToolBar.new(top_dock_site, tool_bar_shell, PACK_UNIFORM_WIDTH)
    FXToolBarGrip.new(project_bar, :opts => TOOLBARGRIP_SINGLE)
    @new_project_button = FXButton.new(project_bar, "\tNew Project\tNew Project." , :icon => ICON_ADD_PROJECT)
    @new_project_button.connect(SEL_COMMAND, method(:onNewProject))
    
    @start_scan_button = FXButton.new(project_bar, "\tStart Scan\tStart Scan." ,:icon => ICON_START)
    @start_scan_button.disable
    # @start_scan_button.tipText = "Start Scan"
    @start_scan_button.connect(SEL_COMMAND, method(:startFullScan))
    
    @pause_scan_button = FXButton.new(project_bar, "\tPause Scan\tPause Scan." ,:icon => ICON_PAUSE)
    @pause_scan_button.disable
    # @start_scan_button.tipText = "Start Scan"
    @pause_scan_button.connect(SEL_COMMAND, method(:pauseScan))
    
    
    views_bar = FXToolBar.new(top_dock_site, tool_bar_shell, :opts => PACK_UNIFORM_WIDTH)
    FXToolBarGrip.new(views_bar, :opts => TOOLBARGRIP_SINGLE)
    
    #@create_report_button = FXButton.new(views_bar,"\tReport\tCreate Report." , :icon => @reportIcon)
    #@create_report_button.disable
    
    @show_dashboard_button = FXButton.new(views_bar, "\tShow Dashboard\tShow Dashboard.", :icon => ICON_DASHBOARD)
    @show_dashboard_button.connect(SEL_COMMAND, method(:showDashboard))
    # @create_report_button.disable
    @show_conversation_button = FXButton.new(views_bar, "\tShow Conversation\tShow Conversation.", :icon => ICON_CONVERSATION)
    @show_conversation_button.connect(SEL_COMMAND, method(:showConversation))
    
    tools_bar = FXToolBar.new(top_dock_site, tool_bar_shell, :opts => PACK_UNIFORM_WIDTH)
    FXToolBarGrip.new(tools_bar, :opts => TOOLBARGRIP_SINGLE)
    
    @open_transcoder_button = FXButton.new(tools_bar, "\tOpen Transcoder\tOpen Transcoder.", :icon => ICON_TRANSCODER)
    @open_transcoder_button.connect(SEL_COMMAND, method(:onOpenTranscoder))
    
    @statusBar = StatusBar.new(self, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM, :padding => 0)
    
    # statusInfo = FXHorizontalFrame.new(quickInfo, :opts => FRAME_SUNKEN, :padding => 0)
    # FXLabel.new(statusInfo, "Status: ")
    # @statusInfo = FXLabel.new(statusInfo, "- no project started -")
    
    # frame = FXHorizontalFrame.new(quickInfo, :opts => FRAME_SUNKEN, :padding => 0)
    # FXLabel.new(frame, "Project: ")
    # @projectInfo = FXLabel.new(frame, " - ")
    
    # frame = FXHorizontalFrame.new(quickInfo, :opts => FRAME_SUNKEN, :padding => 0)
    # FXLabel.new(frame, "Session: ")
    # @sessionInfo = FXLabel.new(frame, " - ")
    
    # frame = FXHorizontalFrame.new(quickInfo, :opts => FRAME_SUNKEN, :padding => 0)
    # FXLabel.new(frame, "Port: ")
    # @portInfo = FXLabel.new(frame, " - ")
    
    # Main contents area is split left-to-right
    splitter = FXSplitter.new(self, :opts => LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING)
    
    # tree_frame = FXVerticalFrame.new(splitter,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :width => 300, :padding=>0)
    #@shutter = FXShutter.new(splitter, :opts => FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, :width => 300, :padding => 0, :hSpacing => 0, :vSpacing => 0)
    
    #findingsShutterItem = FXShutterItem.new(@shutter, "Findings", nil, :opts => LAYOUT_FILL_Y|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT, :padding => 1, :hSpacing => 1, :vSpacing => 1)  
    #sitesShutterItem = FXShutterItem.new(@shutter, "Sites", nil, :opts => LAYOUT_FILL_Y|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT, :padding => 1, :hSpacing => 1, :vSpacing => 1) 
    
    # @findings_tree = FXTreeList.new(findingsShutterItem.content, :opts => (LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_RIGHT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES|TREELIST_EXTENDEDSELECT))
    #icon_list = ""
    #@findings_tree = Watobo::Gui::FindingsTree.new(findingsShutterItem.content, self, nil)                                                                       
    
    #@sites_tree = Watobo::Gui::SitesTree.new(sitesShutterItem.content, self, nil)
    
    frame = FXVerticalFrame.new(splitter, :opts => LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH|FRAME_SUNKEN, :padding => 0, :width => 200)
    @treeTabbook = FXTabBook.new(frame, nil, 0, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
    
    docfilter_tab = FXTabItem.new(@treeTabbook, "Findings", nil)
    tab_frame = FXVerticalFrame.new(@treeTabbook, :opts => LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|FRAME_RAISED, :width => 100)
    frame = FXVerticalFrame.new(tab_frame, :opts => LAYOUT_FILL_Y|LAYOUT_FILL_X|FRAME_SUNKEN, :padding => 0)
    @findings_tree = Watobo::Gui::FindingsTree.new(frame, self, nil)
    docfilter_tab = FXTabItem.new(@treeTabbook, "Sites", nil)
    tab_frame = FXVerticalFrame.new(@treeTabbook, :opts => LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|FRAME_RAISED, :width => 100)
    frame = FXVerticalFrame.new(tab_frame, :opts => LAYOUT_FILL_Y|LAYOUT_FILL_X|FRAME_SUNKEN, :padding => 0)
    @sites_tree = Watobo::Gui::SitesTree.new(frame, self, nil)
    #
    # S W I T C H E R
    #
    
    @switcher = FXSwitcher.new(splitter,LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    
    # R E Q U E S T I N F O
    requestInfo = FXVerticalFrame.new(@switcher, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    request_splitter = FXSplitter.new(requestInfo, :opts => LAYOUT_SIDE_TOP|LAYOUT_FILL_X|SPLITTER_HORIZONTAL|LAYOUT_FILL_Y|SPLITTER_TRACKING)
    
    listFrame = FXVerticalFrame.new(request_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN)
    
    # filterFrame =FXVerticalFrame.new(listFrame, LAYOUT_FILL_X)
    
    @tabBook = FXTabBook.new(listFrame, nil, 0, :opts => LAYOUT_FILL_X|LAYOUT_RIGHT)
    
    docfilter_tab = FXTabItem.new(@tabBook, "Doc Filter", nil)
    
    docfilter_frame = FXHorizontalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
    
    @foption_nopix = FXCheckButton.new(docfilter_frame, "excl. pics", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
    @foption_nopix.setCheck(true)
    @foption_nodocs = FXCheckButton.new(docfilter_frame, "excl. docs", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
    @foption_nodocs.setCheck(true)
    @foption_nojs = FXCheckButton.new(docfilter_frame, "excl. javascript", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
    @foption_nojs.setCheck(true)
    @foption_nocss = FXCheckButton.new(docfilter_frame, "excl. style sheets", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
    @foption_nocss.setCheck(true)
    
    search_tab = FXTabItem.new(@tabBook, "Text Filter", nil)
    
    search_frame = FXHorizontalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
    FXButton.new(search_frame, "Clear", nil, nil, 0, FRAME_RAISED|FRAME_THICK).connect(SEL_COMMAND, method(:onClear))
    @tableFilterFX = FXTextField.new(search_frame, 20, @table_filter, FXDataTarget::ID_VALUE, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X)
    # filterOptionsFrame =FXHorizontalFrame.new(fbox, LAYOUT_FILL_X)
    @foption_req = FXCheckButton.new(search_frame, "Full Request", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
    @foption_res = FXCheckButton.new(search_frame, "Full Response", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
    
    options_tab = FXTabItem.new(@tabBook, "Options", nil)    
    options_frame = FXHorizontalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
    @table_option_autoscroll = FXCheckButton.new(options_frame, "autoscroll", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
    @table_option_autoscroll.setCheck(false)
    
    @table_option_unique = FXCheckButton.new(options_frame, "unique chats", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
    @table_option_unique.setCheck(false)
    
    
    
    #applyFilterButton = FXButton.new(listFrame, "Apply", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
    docfilter_button_frame = FXHorizontalFrame.new(listFrame, :opts => LAYOUT_FILL_X)
    FXButton.new(docfilter_button_frame, "Apply", nil, nil, 0, FRAME_RAISED|FRAME_THICK).connect(SEL_COMMAND, method(:onApplyFilterClick))
    #FXButton.new(docfilter_button_frame, "Clear", nil, nil, 0, FRAME_RAISED|FRAME_THICK).connect(SEL_COMMAND, method(:onClear))
    
    @table_filter.connect(SEL_COMMAND) do |sender, sel, event|
      # puts "apply filter #{@table_filter.value}"
      applyFilter()
    end
    
    #==============================================================================================#
    # Chat-Table
    #==============================================================================================#
    
    #@chatTable = FXTable.new(listFrame,
    @chatTable = ConversationTable.new(listFrame,
                                       :opts => TABLE_COL_SIZABLE|TABLE_ROW_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y|TABLE_READONLY,
                                       :padding => 2)
    
    @table_option_autoscroll.connect(SEL_COMMAND) {
      @chatTable.autoscroll =  true if @table_option_autoscroll.checked?
    }
    
    @chatTable.connect(SEL_COMMAND) do |sender, sel, item|
      @findings_tree.killSelection()
      @sites_tree.killSelection()
      onTableClick(sender,sel,item)
    end
    
    @chatTable.connect(SEL_DOUBLECLICKED) do |sender, sel, data|
      @findings_tree.killSelection()
      @sites_tree.killSelection()
      row = sender.getCurrentRow
      if row >= 0 then
        @chatTable.selectRow(row, false)
        chatid=@chatTable.getRowText(row).to_i
        chat = @active_project.getChat(chatid)
        open_manual_request_toolkit(chat)
      end
    end
    
    @chatTable.connect(SEL_RIGHTBUTTONRELEASE) do |sender, sel, event|
      @findings_tree.killSelection()
      @sites_tree.killSelection()
      unless event.moved?
        #   row = sender.getCurrentRow
        ypos = event.click_y
        row = @chatTable.rowAtY(ypos)
        
        
        if @chatTable.numRows > 0 and row >= 0 then
          
          @chatTable.selectRow(row, false)
          chatid=@chatTable.getRowText(row).to_i
          
          FXMenuPane.new(self) do |menu_pane|
            doManual = FXMenuCommand.new(menu_pane, "manual request(#{chatid}) ..." )
            fuzzIt = FXMenuCommand.new(menu_pane, "start fuzzer(#{chatid}) ..." )
            addToLogin = FXMenuCommand.new(menu_pane, "add chat(#{chatid}) to Login-Script ..." )
            removeExcluded = FXMenuCommand.new(menu_pane, "include chat(#{chatid})" )
            addExcluded = FXMenuCommand.new(menu_pane, "exclude chat(#{chatid})" )
            copyURL = FXMenuCommand.new(menu_pane, "copy URL")
            if @active_project.excluded_chats.include?(chatid) then
              removeExcluded.enable
              addExcluded.disable
            else
              removeExcluded.disable
              addExcluded.enable
            end
            
            copyURL.connect(SEL_COMMAND) {
              url = @active_project.getChat(chatid).request.url
              puts url
              @clipboard.paste(url)
            }
            #  copyRequest = FXMenuCommand.new(menu_pane, "copy Request(#{chatid})" )
            #  copyResponse = FXMenuCommand.new(menu_pane, "copy Response(#{chatid})" )
            
            #  info = FXMenuCommand.new(menu_pane, "Details..." )
            #info.connect(SEL_COMMAND) { display_info_for(item) }
            addToLogin.connect(SEL_COMMAND) { 
              @active_project.loginscript_ids.push chatid 
            }
            
            fuzzIt.connect(SEL_COMMAND) {
              chat = @active_project.getChat(chatid)
              openFuzzer(chat)
            }
            
            doManual.connect(SEL_COMMAND) {
              chat = @active_project.getChat(chatid)
              open_manual_request_toolkit(chat)
            }
            
            addExcluded.connect(SEL_COMMAND){
              @active_project.excluded_chats.push chatid
            }
            
            removeExcluded.connect(SEL_COMMAND){
              @active_project.excluded_chats.delete(chatid)
            }
            
            
            menu_pane.create
            menu_pane.popup(nil, event.root_x, event.root_y)
            app.runModalWhileShown(menu_pane)
            
          end
          
        end
        
      end
      
    end
    
    
    #===================================================================
    # CHAT VIEWER
    #===================================================================
    chatFrame = FXVerticalFrame.new(request_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN, :padding => 0)
    
    view_menu = FXVerticalFrame.new(chatFrame, :opts => LAYOUT_FILL_X, :padding => 0)
    frame = FXHorizontalFrame.new(view_menu, :opts => LAYOUT_FILL_X)
    #cvlabel = FXLabel.new(frame, "View: ")
    # cvlabel.setFont(FXFont.new(getApp(), "helvetica", 12, FONTWEIGHT_BOLD, FONTSLANT_ITALIC, FONTENCODING_DEFAULT))
    
    @viewContent = FXLabel.new(frame, " -N/A- ")
    @viewContent.setFont(FXFont.new(getApp(), "helvetica", 12, FONTWEIGHT_BOLD, FONTSLANT_ITALIC, FONTENCODING_DEFAULT))
    
    frame = FXHorizontalFrame.new(view_menu, :opts => LAYOUT_FILL_X)
    mr_button = FXButton.new(frame, "Manual Request", ICON_MANUAL_REQUEST_MEDIUM, nil, 0, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)
    mr_button.connect(SEL_COMMAND) {
      open_manual_request_toolkit(@lastViewed) if @lastViewed
    }
    
    fuzz_button = FXButton.new(frame, "Fuzzer", ICON_FUZZER_MEDIUM, nil, 0, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)
    fuzz_button.connect(SEL_COMMAND) {
      openFuzzer(@lastViewed) if @lastViewed
    }
    
    @browserView_btn = FXButton.new(frame, "Browser-View", ICON_BROWSER_MEDIUM, nil, 0, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)
    @browserView_btn.connect(SEL_COMMAND) {
      begin
        if @lastViewed and @browserView then
          openBrowser(@lastViewed.request, @lastViewed.response) 
        end
      rescue => bang
        puts bang
        
      end
    }
    
    
    FXHorizontalSeparator.new(chatFrame, :opts => SEPARATOR_GROOVE|LAYOUT_FILL_X)
    #  FXLabel.new(view_menu, "Source:")
    
    
    title_frame = FXHorizontalFrame.new(chatFrame, :opts => LAYOUT_FILL_X)
    FXLabel.new(title_frame, "Request:")
    
    @request_viewer = Watobo::Gui::RequestViewer.new(chatFrame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    #  @request_viewer = Watobo::Gui::ChatViewer.new(chatFrame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    # @request_viewer.highlight_style = 1
    
    FXHorizontalSeparator.new(chatFrame, :opts => SEPARATOR_GROOVE|LAYOUT_FILL_X)
    title_frame = FXHorizontalFrame.new(chatFrame, :opts => LAYOUT_FILL_X)
    FXLabel.new(title_frame, "Response:")
    
    #fxViewButton = FXButton.new(title_frame, "View", nil, nil, 0, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT)
    #fxViewButton.connect(SEL_COMMAND, method(:onViewResponse))
    
    
    # @response_viewer = Watobo::Gui::ChatViewer.new(chatFrame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    @response_viewer = Watobo::Gui::ResponseViewer.new(chatFrame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    # @response_viewer.highlight_style = 2
    
    
    #===================================================================
    # L O G I N F O
    #===================================================================
    logFrame = FXVerticalFrame.new(@switcher, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN)
    FXLabel.new(logFrame, "Eventlist:", :opts => LAYOUT_FILL_X)
    sunkenlogFrame = FXHorizontalFrame.new(logFrame, :opts => FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    @logText = FXText.new(sunkenlogFrame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_AUTOSCROLL)
    
    
    
    # DASHBOARD#
    @dashboard = Dashboard.new(@switcher)
    
    # FINDING INFORMATION
    frame = FXVerticalFrame.new(@switcher, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
    @finding_info = FindingInfo.new(frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
    if @foption_nopix.checked? then @doctype_TableFilter.concat(@fext_pix);end
    if @foption_nodocs.checked? then @doctype_TableFilter.concat(@fext_docs);end
    if @foption_nojs.checked? then @doctype_TableFilter.concat(@fext_javascript);end
    if @foption_nocss.checked? then @doctype_TableFilter.concat(@fext_style);end 
  end
  
  # !!!
  #  TODO: FXRUBY-Bug???
  #  If splash screen is shown app will crash on close :(
  # !!!
  # def create
  #   super
  # splash = FXSplashWindow.new(interface, ICON_SPLASH, SPLASH_SHAPED|SPLASH_DESTROY|SPLASH_OWNS_ICON, 2000)
  #splash = FXSplashWindow.new(@app, ICON_SPLASH, SPLASH_SIMPLE, 2000)
  # splash.execute
  # splash.create()
  #splash.show(PLACEMENT_SCREEN)
  # show(PLACEMENT_SCREEN)
  #show(PLACEMENT_MAXIMIZED)
  
  # end
end