# .
# qManualRequest.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/qRequest.rb'
require 'lib/qShapers.rb'
require 'lib/guiUtils.rb'
require 'lib/qProject.rb'
require 'lib/chatviewer.rb'
require 'lib/requesteditor.rb'
require 'lib/core/scanner'
require 'lib/gui/chat_diff'

class FXQuickScanDialog < FXDialogBox
  attr :am_selector
  attr :excludedChats
  attr :logoutSignature
  
  def initialize(owner, project)
    # Invoke base class initialize function first
    super(owner, "Start Quick Scan", DECOR_TITLE|DECOR_BORDER)
    m = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    title_frame = FXVerticalFrame.new(m, :opts => FRAME_GROOVE|LAYOUT_FILL_X)
    @title = FXLabel.new(title_frame, "", :opts => LAYOUT_FILL_X|JUSTIFY_LEFT)
    main = FXVerticalFrame.new(m, :opts => FRAME_GROOVE|LAYOUT_FILL_X)
    
    @am_selector = Hash.new
    @filter_selector = Hash.new
    
    # select checks
    @title.text = "Select Checks"
    am_select = FXVerticalFrame.new(main, FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    # Bottom buttons
    buttons = FXHorizontalFrame.new(main, :opts => LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,
                                    :padLeft => 40, :padRight => 40, :padTop => 20, :padBottom => 20)
    
    project.active_modules.each do |am|
      rbs = FXDataTarget.new
      cb = FXCheckButton.new(am_select, am.check_name, rbs, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
      @am_selector[am.check_name] = cb 
      cb.setCheck(TRUE)
    end
    
    
    # Accept
    accept = FXButton.new(buttons, "&Accept", nil, self, ID_ACCEPT,
    FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
    
    # Cancel
    FXButton.new(buttons, "&Cancel", nil, self, ID_CANCEL,
    FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
    
    
  end
  
end

class HistoryItem
  attr :id
  attr :request
  attr :response
  attr :chat
  
  def initialize(request, response, history_id)
    @id = history_id
    @request = request
    @response = response
    @chat = Qchat.new(@request, @response, @id)
  end
end

class HistoryButton < FXVerticalFrame
  attr :item
  
  def reset()
    self.backColor = @backColor  
  end
  
  def highlight()
    self.backColor = FXColor::Red
  end
  
  def update(history_item)
    @button.text = history_item.id.to_s
    #@button.text = "Huhule"
    #puts "!update Button #{@button}"
    @button.update()
    @item = history_item
  end
  
  def initialize(owner, text, target, opts)
    super(owner, opts)
    @item = nil
    @backColor = self.backColor
    
    @button = FXButton.new(self, text, nil, target, DiffFrame::ID_HISTORY_BUTTON, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
    
  end
end

class HistorySlider < FXScrollWindow
  def update(history)
    @history = history
    reset()
    history.length.times do |i|
      begin
        
        @buttons[i].update(history[i])
      rescue => bang
        puts bang
      end
    end
  end
  
  def getItem(id)
    item = nil
    @history.each do |h|
      item = h if h.id == id
      break if item    
    end
    return item
  end
  
  def reset()
    @buttons.each do |b|
      b.reset()
    end
    
  end
  
  def initialize(owner, target, history_size, opts)
    @size = history_size
    @buttons = []
    @history = []
    super(owner, opts)   
    frame = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_Y|LAYOUT_FILL_X, :padding => 0)
    @size.times do |i|
      @buttons.push HistoryButton.new(frame, 'empty', target, :opts => FRAME_NONE|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, :width => 50, :height => 50)
    end
  end
end

class DiffFrame < FXVerticalFrame
  include Responder
  ID_HISTORY_BUTTON = FXMainWindow::ID_LAST
  
  def onHistoryButton(sender, sel, event)
    @history_slider.reset()
    sender.parent.backColor = FXColor::Red
    @slider_selection = sender.parent.item
  end
  
  def updateHistory(history)
    @slider_selection = nil
    @history = history
    @history_slider.update(history)
    updateHistoryTable()
    updateSelections()
  end
  
  def initHistoryTable()
    @historyTable.clearItems()
    @historyTable.setTableSize(0, 3)
    
    @historyTable.setColumnText( 0, "STATUS" ) 
    @historyTable.setColumnText( 1, "LENGTH" )
    @historyTable.setColumnText( 2, "URL" )
    
    @historyTable.rowHeader.width = 50
    @historyTable.setColumnWidth(0, 100)
    
    @historyTable.setColumnWidth(1, 100)
    @historyTable.setColumnWidth(2, 200)
    
    
  end
  
  def updateHistoryTable()
    begin
      @historyTable.clearItems()
      initHistoryTable()
      
      @history.each do |h|
        lastRowIndex = @historyTable.getNumRows
        @historyTable.appendRows(1)
        @historyTable.setRowText(lastRowIndex, h.id.to_s)
        @historyTable.setItemText(lastRowIndex, 0, h.chat.response.status)
        @historyTable.setItemText(lastRowIndex, 1, h.chat.response.length.to_s)
        @historyTable.setItemText(lastRowIndex, 2, h.chat.request.url)
        3.times do |i|
          @historyTable.getItem(lastRowIndex, i).justify = FXTableItem::LEFT
        end
      end
    rescue => bang
      puts bang
    end
    
  end
  
  def updateSelections()
    
    case @first_chat_dt.value
      when 0
      @first_selection = @history.first
      when 1
      @first_selection = @history.last
      when 2
      @first_selection = @history[-2]
      when 3
      @first_selection = @history_slider.getItem(@first_id_dt.value.to_i) if @first_id_dt.value
    end
    
    case @second_chat_dt.value
      when 0
      @second_selection = @history.first
      when 1
      @second_selection = @history.last
      when 2
      @second_selection = @history[-2]
      when 3
      @second_selection = @history_slider.getItem(@first_id_dt.value.to_i) if @first_id_dt.value
    end
  end
  
  def updateRadioButtons(sender, sel, event)
    @first_last_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
    @first_history_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
    @first_first_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
    @first_prev_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
    
    @second_last_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
    @second_history_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
    @second_first_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
    @second_prev_rb.handle(self, FXSEL(SEL_UPDATE, 0), nil)
    
    @first_history_btn.enabled = @first_history_rb.checked?
    @second_history_btn.enabled = @second_history_rb.checked?
    
    updateSelections()
  end
  
  def getDiffChats()
    first = nil
    second = nil
    begin
      case @first_chat_dt.value
        when 0
        
        when 1
        
      end
      
    rescue

      return first, second 
    end
  end
  
  def initialize(owner, opts)
    super(owner, opts)
    
    @history_size = 10
    @history = []
    @slider_selection = nil
    @first_selection = nil
    @second_selection = nil
    
    FXMAPFUNC(SEL_COMMAND, DiffFrame::ID_HISTORY_BUTTON, 'onHistoryButton')
    
    frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN)
    frame_left = FXVerticalFrame.new(frame, :opts => LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH, :width => 70, :padding => 0)
    @history_slider = HistorySlider.new(frame_left, self, @history_size, opts)
    
    frame_right = FXVerticalFrame.new(frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    sunken = FXVerticalFrame.new(frame_right, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding => 0)
    @historyTable = FXTable.new(sunken, :opts => FRAME_SUNKEN|TABLE_COL_SIZABLE|TABLE_ROW_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y|TABLE_READONLY|LAYOUT_SIDE_TOP, :padding => 2)
    initHistoryTable()
    
    first_chat_gb = FXGroupBox.new(frame_right, "First Chat", LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
    
    @first_chat_dt = FXDataTarget.new(1)  
    @first_first_rb = FXRadioButton.new(first_chat_gb, "First Chat", @first_chat_dt, FXDataTarget::ID_OPTION)
    @first_last_rb = FXRadioButton.new(first_chat_gb, "Last Chat", @first_chat_dt, FXDataTarget::ID_OPTION + 1)
    @first_prev_rb = FXRadioButton.new(first_chat_gb, "Previous Chat", @first_chat_dt, FXDataTarget::ID_OPTION + 2)
    @first_history_rb = FXRadioButton.new(first_chat_gb, "History", @first_chat_dt, FXDataTarget::ID_OPTION + 3) 
    
    @first_chat_dt.connect(SEL_COMMAND, method(:updateRadioButtons))
    
    
    
    frame = FXHorizontalFrame.new(first_chat_gb, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    @first_history_btn = FXButton.new(frame, "get id ")
    @first_id_dt = FXDataTarget.new('')
    @first_id = FXTextField.new(frame, 1, :target => @first_id_dt, :selector => FXDataTarget::ID_VALUE, :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN)
    @first_history_btn.connect(SEL_COMMAND) {
      if not @slider_selection.nil?
        @first_id_dt.value = @slider_selection.id
        @first_id.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @first_selection = @slider_selection
      end
    }
    
    
    second_chat_gb = FXGroupBox.new(frame_right, "Second Chat", LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
    
    @second_chat_dt = FXDataTarget.new(1)
    @second_chat_dt.connect(SEL_COMMAND, method(:updateRadioButtons))  
    @second_first_rb = FXRadioButton.new(second_chat_gb, "First Chat", @second_chat_dt, FXDataTarget::ID_OPTION)
    @second_last_rb = FXRadioButton.new(second_chat_gb, "Last Chat", @second_chat_dt, FXDataTarget::ID_OPTION + 1)
    @second_prev_rb = FXRadioButton.new(second_chat_gb, "Previous Chat", @second_chat_dt, FXDataTarget::ID_OPTION + 2)    
    @second_history_rb = FXRadioButton.new(second_chat_gb, "History", @second_chat_dt, FXDataTarget::ID_OPTION + 3)
    
    frame = FXHorizontalFrame.new(second_chat_gb, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    @second_history_btn = FXButton.new(frame, "get id ")
    @second_id_dt = FXDataTarget.new('')
    @second_id = FXTextField.new(frame, 1, :target => @second_id_dt, :selector => FXDataTarget::ID_VALUE, :opts => TEXTFIELD_NORMAL|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN)
    
    @second_history_btn.connect(SEL_COMMAND) {
      if not @slider_selection.nil?
        @second_id_dt.value = @slider_selection.id
        @second_id.handle(self, FXSEL(SEL_UPDATE, 0), nil)
        @second_selection = @slider_selection
      end
    }
    
    diff_button = FXButton.new(frame_right, "Diff it!", nil, nil, 0, :opts => LAYOUT_FILL_X|FRAME_RAISED|FRAME_THICK)
    
    diff_button.connect(SEL_COMMAND) {      
      # new, orig = getDiffChats()
      if @first_selection and @second_selection then
        orig = @first_selection.chat
        new = @second_selection.chat      
        project = nil
        diffViewer = ChatDiffViewer.new(FXApp.instance, project, orig, new)
        diffViewer.create
        diffViewer.show(Fox::PLACEMENT_SCREEN)
      end
    }
  end
end

# 
class QManualRequestSender < Watobo::Framework::Session
  def initialize(project)
    super(project)
  end
  
  def runLogin(prefs)
    @project.loginscript_ids.each do |chatid|
      #     puts "login request #{chatid}"
      test_req = @project.getChat(chatid).copyRequest
      
      #  puts test_req
      
      request,response = doRequest(test_req,:update_sids => true, :update_session => true, :update_contentlength => true)
      puts 
      #   puts "+ Login-Request:"
      #  puts request
      #   puts
      #   puts "+ Login-Response:"
      #   puts response
    end
  end
  
  
  def sendRequest(new_request, prefs)
    
    if prefs[:update_session ] == true and prefs[:run_login ] == true
      runLogin(prefs)
    end
    new_request.extend Watobo::Parser::Web10
    new_request.extend Watobo::Framework::Shaper::Web10
    begin
      test_req, test_resp = self.doRequest(new_request, prefs)
      
      if prefs[:log_chat ] == true
        chat = Qchat.new(test_req, test_resp, @project.last_chatid + 1)
        @project.addChat(chat)
      end
    rescue => bang
      puts bang
    end
    return test_req,test_resp
  end
end

#
#--------------------------------------------------------------------------------------------
#
class QXManualRequest < FXDialogBox
  
  include Watobo::Gui::Utils
  include Watobo::Core
  
  def onRequestReset(sender,sel,item)
    @req_builder.setText(@original_request.join.gsub(/\r/,""))
  end
  
  def onShowPreview(sender, sel, item)
    @interface.showPreview(request, response)  
  end
  
  def logger(message)
    t = Time.now  
    now = t.strftime("%m/%d/%Y @ %H:%M:%S")  
    @log_viewer.insertText(0, "#{now}: #{message}\n")
    
  end
  
  def addHistoryItem(request, response)
    @counter += 1
    @history.push HistoryItem.new(@last_request, @last_response, @counter)  
    
    @history.shift if @history.length > @history_size
    
    @diff_frame.updateHistory(@history)
  end
  
  def onBtnQuickScan(sender, sel, item)
    dlg = Watobo::Gui::QuickScanDialog.new(self, "Select ActiveChecks", @project.active_modules, @project.settings[:active_policies], @project.settings[:active_policy])
    if dlg.execute != 0 then
      scan_modules = []
      scan_chats = []    
      
      if @useOriginalRequest.checked? then
        req = @request
      else
        req = @req_builder.parseRequest()
      end
      scan_chats.push Qchat.new(req, [""], 0)
      pblist = Hash.new # hash for storing progressbar information
      
      @project.settings[:active_policy] = dlg.active_policy
      @project.active_modules.each do |m|
        if m.enabled? then
          scan_modules.push m
          # setup progress bars
          pblist[m] = @pbar
        else
          puts "skipping #{m.info[:check_name]}..."
        end
      end
      puts "Create Scanner ..."
      scanner = Watobo::Core::Scanner.new(@project, self, scan_modules, scan_chats)
      puts "Checks Total: #{scanner.numTotalChecks}"
      @pbar.total = scanner.numTotalChecks
      @pbar.progress = 0
      @pbar.barColor = FXRGB(255,0,0)
      
      Thread.new {
        begin
          scanner.run(:progress_displays => pblist)
          logger("scan finished")
          @pbar.total = 0
          @pbar.progress = 0
          @pbar.barColor = 'grey'
        rescue => bang
          puts "Error: ScanRun"
          puts bang
          puts bang.backtrace.join("\n")
          raise
        end
      }
    end
    
    return 0
    
  end
  
  def onBtnSendClick(sender,sel,item)
    @request_viewer.setText('')
    @response_viewer.setText('')
    new_request = @req_builder.parseRequest
    logger("sending request") 
    
    Thread.new(new_request) { |test_request|
      begin
        @last_request, @last_response = @request_sender.sendRequest(test_request, 
                                                                    :run_login => @runLogin.checked?, 
                                                                    :update_session => @updateSession.checked?, 
                                                                    :update_contentlength => @updateContentLength.checked?,
                                                                    :log_chat => @logChat.checked?)
      rescue => bang
        puts bang
        
      end
      
      if @last_request then 
        logger("got answer")
        @request_viewer.setText(@last_request)
        
        @response_viewer.setText(@last_response)
        @responseMD5.text = @last_response.contentMD5
        
        addHistoryItem(@last_request, @last_response)
        
      else
        logger("ERROR: #{@last_response.first}");
        @responseMD5.text = "- N/A -"
      end
      
      
      
    }
  end
  
  
  
  def onPreviewClick(sender,sel,item)
    @request_viewer.setText('')
    new_request = @req_builder.parseRequest
    #  puts "new request: #{new_request}"
    @request_viewer.setText(new_request) 
    @tabBook.current = 1
  end
  
  #def a_onReqChanged(sender,sel,item)
  #  pattern="(%%.*%%)"
  #  highlight_pattern(@req_builder, pattern)
  #end
  
  #def a_highlight_pattern(text_box, pattern)
  #  res_text = text_box.to_s
  #  begin
  #    if res_text =~ /(#{pattern})/i then
  #      if $1 then
  #        string = $1
  #      else
  #        string = pattern
  #      end
  #      #  puts "highlighting: #{string}"
  #      if res_text.index(string) then
  #        spos = 0
  #        while pos = res_text.index(string, spos)
  #          # puts pos
  #          spos = pos + string.length
  #          text_box.changeStyle(pos,string.length,1)
  #        end
  #      end      
  #    end
  #    
  #  rescue
  #    @log_viewer.appendText "Not a valid regex! (#{pattern})\n"
  #  end
  #  
  #end
  
  
  #def initialize(app, project, chat)
  
  def initialize(owner, project, chat)
    # Invoke base class initialize function first
    super(owner, "Manual Request Toolkit", :opts => DECOR_ALL,:width=>800, :height=>600)
    @request_sender = QManualRequestSender.new(project)
    @responseFilter = FXDataTarget.new("")
    
    @chat = chat
    
    @original_request = chat.request.dup
    @project = project
    @interface = project.interface
    
    @request = chat.request.dup
    
    @tselect = ""
    @sel_pos = ""
    @sel_len = ""
    
    @last_request = []
    @last_response = []
    
    @history_size = 10
    @history = []
    @counter = 0
    
    self.icon = ICON_MANUAL_REQUEST
    
    # Construct some hilite styles
    hs_red = FXHiliteStyle.new
    hs_red.normalForeColor = FXRGBA(255,255,255,255) # FXColor::Red
    hs_red.normalBackColor = FXRGBA(255,0,0,1)   # FXColor::White
    hs_red.style = FXText::STYLE_BOLD
    
    
    
    mr_splitter = FXSplitter.new(self, LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_VERTICAL|SPLITTER_REVERSED|SPLITTER_TRACKING)
    # top = FXHorizontalFrame.new(mr_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_SIDE_BOTTOM)
    top_frame = FXVerticalFrame.new(mr_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y||LAYOUT_FIX_HEIGHT|LAYOUT_BOTTOM,:height => 500) 
    top_splitter = FXSplitter.new(top_frame, LAYOUT_FILL_X|SPLITTER_HORIZONTAL|LAYOUT_FILL_Y|SPLITTER_TRACKING)
    
    log_frame = FXVerticalFrame.new(mr_splitter, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM,:height => 100)
    
    #LAYOUT_FILL_X in combination with LAYOUT_FIX_WIDTH
    
    req_editor = FXVerticalFrame.new(top_splitter, :opts => LAYOUT_FILL_X|LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y|FRAME_GROOVE,:width=>400, :height=>500)
    
    req_edit_header = FXHorizontalFrame.new(req_editor, :opts => LAYOUT_FILL_X)
    FXLabel.new(req_edit_header, "Request Editor:" )
    req_viewer = FXVerticalFrame.new(req_editor, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding=>0)
    @req_builder = Watobo::Gui::RequestEditor.new(req_viewer, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    @req_builder.setText(@request.join.gsub(/\r/,""))
    
    req_reset_button = FXButton.new(req_edit_header, "Reset", nil, nil, 0, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT)
    req_reset_button.connect(SEL_COMMAND, method(:onRequestReset))
    
    
    frame = FXHorizontalFrame.new(req_editor, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM, :padding => 0)
    req_options = FXVerticalFrame.new(frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    #eq_options = FXVerticalFrame.new(frame, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM)
    opt = FXGroupBox.new(req_options, "Request Options", LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
    
    #  opt = FXVerticalFrame.new(frame,:opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    #  btn = FXVerticalFrame.new(frame,:opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    #FXCheckButton.new(rob, "URL Encoding", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    @updateContentLength = FXCheckButton.new(opt, "Update Content-Length", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    @updateContentLength.checkState = true
    
    @updateSession = FXCheckButton.new(opt, "Update Session Information", nil, 0, JUSTIFY_LEFT|JUSTIFY_TOP|ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    @updateSession.checkState = true
    @updateSession.connect(SEL_COMMAND) do |sender, sel, item|
      @runLogin.enabled = @updateSession.checked?
    end
    
    
    @runLogin = FXCheckButton.new(opt, "Run Login", nil, 0, JUSTIFY_LEFT|JUSTIFY_TOP|ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    @runLogin.checkState = false
    
    @useOriginalRequest = FXCheckButton.new(opt, "Use Original Request (QuickScan)", nil, 0, JUSTIFY_LEFT|JUSTIFY_TOP|ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    @useOriginalRequest.checkState = true
    
    @logChat = FXCheckButton.new(opt, "Log Chat", nil, 0,
    ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    @logChat.checkState = false
    
    button_frame = FXVerticalFrame.new(frame, :opts => LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH|LAYOUT_RIGHT, :width => 100)
    send_frame = FXVerticalFrame.new(button_frame, :opts => LAYOUT_FILL_Y|LAYOUT_FILL_X, :padding => 2)
    send_frame.backColor = FXColor::Red
    #btn_send = FXButton.new(frame, "\nSEND", ICON_SEND_REQUEST, nil, 0, :opts => ICON_ABOVE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH|LAYOUT_RIGHT, :width => 100)
    btn_send = FXButton.new(send_frame, "\nSEND", ICON_SEND_REQUEST, nil, 0, :opts => ICON_ABOVE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_Y|LAYOUT_FILL_X|LAYOUT_RIGHT)
    btn_prev = FXButton.new(button_frame, "preview >>", nil, nil, 0, :opts => LAYOUT_FILL_X|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT)
    btn_prev.connect(SEL_COMMAND,method(:onPreviewClick))
    
    frame = FXHorizontalFrame.new(req_editor, :opts => LAYOUT_FILL_X|FRAME_GROOVE)
    
    btn_quickscan = FXButton.new(frame, "QuickScan", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
    btn_quickscan.connect(SEL_COMMAND, method(:onBtnQuickScan))
    @pbar = FXProgressBar.new(frame, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_HORIZONTAL)
    #@pbar.create
    @pbar.connect(SEL_CHANGED) {
      print ":"
    }
    @pbar.progress = 0
    @pbar.total = 0
    @pbar.barColor=0
    @pbar.barColor = 'grey' #FXRGB(255,0,0)
    
    
    
    
    # TODO: Implement font sizing
    #@req_builder.font = FXFont.new(app, "courier" , 14, :encoding=>FONTENCODING_ISO_8859_1)
    
    result_viewer = FXVerticalFrame.new(top_splitter, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE|LAYOUT_FIX_WIDTH, :width => 400)
    
    # log_viewer = FXVerticalFrame.new(bottom_frame, :opts => LAYOUT_FILL_X|FRAME_GROOVE|LAYOUT_BOTTOM)
    
    @tabBook = FXTabBook.new(result_viewer, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
    
    resp_tab = FXTabItem.new(@tabBook, "Response", nil)
    frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
    @response_viewer = Watobo::Gui::ResponseViewer.new(frame, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
   # @response_viewer.max_len = 0
    
    options = FXHorizontalFrame.new(frame, :opts => LAYOUT_FILL_X, :padding => 0)
    frame = FXHorizontalFrame.new(options, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN, :padding => 0)
    frame.backColor = FXColor::White
    label = FXLabel.new(frame, "MD5: ", :opts => LAYOUT_FILL_Y|JUSTIFY_CENTER_Y)
    label.backColor = FXColor::White
    @responseMD5 = FXLabel.new(frame, "-N/A-", :opts => LAYOUT_FILL_Y|JUSTIFY_CENTER_Y)
    @responseMD5.backColor = FXColor::White
    
    browser_button = FXButton.new(options, "Browser-View", ICON_BROWSER_MEDIUM, nil, 0, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)
    browser_button.connect(SEL_COMMAND) {
      begin
        if @last_request and @last_response then
          @interface.openBrowser(@last_request, @last_response) 
        end
      rescue => bang
        puts bang
        
      end
    }
    
    
    req_tab = FXTabItem.new(@tabBook, "Request", nil)   
    @request_viewer = Watobo::Gui::RequestViewer.new(@tabBook, FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
    diff_tab = FXTabItem.new(@tabBook, "Differ", nil)   
    
    @diff_frame = DiffFrame.new(@tabBook, :opts => FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
    log_frame_header = FXHorizontalFrame.new(log_frame, :opts => LAYOUT_FILL_X)
    FXLabel.new(log_frame_header, "Logs:" )
    
    
    #log_text_frame = FXHorizontalFrame.new(bottom_frame, :opts => LAYOUT_FILL_X|FRAME_SUNKEN|LAYOUT_BOTTOM)
    log_text_frame = FXVerticalFrame.new(log_frame, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding=>0)
    @log_viewer = FXText.new(log_text_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y)
    #--------------------------------------------------------------------------------
    
    
    btn_send.connect(SEL_COMMAND, method(:onBtnSendClick))
    
    
    
  end
  
  
  
end