# .
# qProject.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/httpparser.rb'
require 'lib/qShapers.rb'
#require 'lib/qConnection.rb'
require 'lib/qRequest.rb'
require 'zlib'
require 'lib/core/passive_check.rb'  
require 'lib/core/scanner'

class Qchat
  attr :request
  attr :response
  attr :id
  attr :comment
  
  
  public
  
  def copyRequest()
    req_copy = []
    self.request.each do |line|
      req_copy.push line.clone
    end
    # now extend the new request with the Watobo mixins
    req_copy.extend Watobo::Parser::Web10
    req_copy.extend Watobo::Framework::Shaper::Web10
    return req_copy
  end
  
  def initialize(request,response,id)
    @request = request.dup
    @response = response.dup
    @id = id
    @comment = ''
    #TODO: Implement type dependent extensions (e.g. Web10, jQUERY, JSON, ...)
    @request.extend Watobo::Parser::Web10
    @response.extend Watobo::Parser::Web10

  end
  
end


class Qfinding
  attr :request
  attr :response
  attr :id
  attr :info
  attr :falsePositive
  
  def initialize(finding_id, info)
    @info = info.dup
    # puts "init finding: #{info[:type]}"
    @request = @info[:chat].request
    @response = @info[:chat].response
    @id = finding_id
    @falsePositive = false
  end
  
end

class QSessionManager < Watobo::Framework::Session
  def initialize(project)
    super(project)
    
  end
  
  def runLogin
    @project.loginscript_ids.each do |chatid|
      puts
      puts "! login request #{chatid}"
      puts 
      test_req = @project.getChat(chatid).copyRequest
      
      # puts test_req
      request, response = doRequest(test_req, :update_sids => true, :update_session => true, :update_contentlength => true)
      #    puts request
      #    puts response
      puts 
      puts "! finished login"
      puts
    end
    # exit
  end
  
  
end

class Qproject
  
  attr :chat_list
  attr :sessionMgr
  #  attr :fext_pix
  #  attr :fext_docs
  #  attr :fext_javascript
  #  attr :fext_style
  attr :date
  
  attr :interface
  attr_accessor :settings
  attr_accessor :numRunningChecks
  
  attr_accessor :valid_sids
  #attr_accessor :proxy
  #attr_accessor :sid_patterns
  attr_accessor :loginscript_ids
  # attr_accessor :run_active
  attr_accessor :active_modules
  attr_accessor :passive_modules
  # attr_accessor :dashboard
  
  # attr_accessor :logout_signature
  # attr_accessor :intercept_port
  
  attr_accessor :excluded_chats
  # attr_accessor :excluded_sites
  #attr_accessor :excluded_urls
  attr_accessor :findings
  
  attr_accessor :intercept_request
  attr_accessor :intercept_response
  attr_accessor :interceptor
  
  # public
  # @project_name
  
  attr :target_filter
  
  include Watobo::Utils
  include Watobo::Core
  
  def uniqueRequestHash(request)
    begin
      parms = request.site + request.method + request.resource
      request.get_parm_names.each do |p|
        if @settings[:smart_scan_parms].include?(p) then
          parms += p + request.get_parm_value(p)
        else
          parms += p
        end
        
      end
      request.post_parm_names.each do |p|
        if @settings[:smart_scan_parms].include?(p) then
          parms += p + request.post_parm_value(p)
        else
          parms += p
        end
        
      end
      return Digest::MD5.hexdigest(parms)
    rescue
      return nil
    end
  end
  
  def last_chatid
    if @chat_list.length > 0
      return @chat_list.last.id
    else
      return 0
    end
  end
  
  def getChat(chatid)
    
    @chat_list.each do |c|     
      if c.id.to_s == chatid.to_s then     
        return c
      end         
    end
    return nil
  end
  
  def addChat(chat)
    chat_list.push chat
    Thread.new(chat) {|c|
      @passive_modules.each do |test_module|
        test_module.do_test(c)       
      end
    }
    
    @interface.addChat(self, chat) if @interface   
    
    
    if @conversations_path != "" then
      request_file = "#{@conversations_path}/#{chat.id}-request"
      response_file = "#{@conversations_path}/#{chat.id}-response"
      if not File.exists?(request_file) then
        fh = File.new(request_file, "w+b")
        fh.print chat.request
        fh.close
        fh = File.new(response_file, "w+b")
        fh.print chat.response
        fh.close
      end
      #    puts "Create File: #{request_file}"
      #    puts "Create File: #{response_file}"
    end
  end
  
  def runLogin
    @sessionMgr.runLogin()    
  end
  
  # TODO: check
  def createScanner() 
    scan_modules = []
    scan_chats = []
    uniqueRequests = Hash.new
    @interface.logger("*** SETTING UP SCANNER ***")
    @interface.logger("Targets:")
    @settings[:target_sites].each do |site|
      @interface.logger("+ #{site}")
    end
    
    @active_modules.each do |test_module|
      if not test_module.enabled? then
        puts "+ skip disabled module #{test_module.info[:check_name]}"
        next
      end
      scan_modules.push test_module
    end
    
    @chat_list.each do |chat|
      if @excluded_chats.include?(chat.id) then
        puts "* [#{chat.id}] skip excluded chat"
        next
      end
      if @excluded_sites.include?(chat.request.site) then
        puts "* [#{chat.id}] skip excluded site: #{chat.request.site}"
        next
      end
      if @settings[:target_sites].include?(chat.request.site) then
        uh = uniqueRequestHash(chat.request)
        if not uniqueRequests.has_key?(uh)
          scan_chats.push chat
          uniqueRequests[uh] = chat
          else
        puts "* skipping chat[#{chat.id}] - hash(#{uh}) already exists"
        end
      end
    end
    
    
    @interface.logger("Selected Modules (#{scan_modules.length}):")
    scan_modules.each do |mod|
      @interface.logger("+ #{mod.info[:check_name]}")      
    end
    @interface.logger("Number of Chats: #{scan_chats.length}")
    scanner = Scanner.new(self, @interface, scan_modules, scan_chats)
  end
  
  def run_passive_checks
    #
    # PASSIVE CHECKS
    #
    @interface.log("====== starting passive checks ======")
    @passive_modules.each do |test_module|
      
      begin 
        
        @interface.log("running checks with module (#{test_module.class})")
        @chat_list.each do |chat|
          
          @dashboard.update_status(test_module, @chat_list.index(chat))if @dashboard
          
          test_module.do_test(chat)         
          
        end
        
      rescue => bang
        @interface.log("problem in module (#{test_module.class})\n#{bang}")
      end
      @interface.log("====== finished passive checks ======")
    end
  end
  
  def saveProjectSettings()
    settings_names = [
    :project_path,
    :session_path,
    :project_name,
    :module_path,
    :intercept_port,
    :proxy_list,
    :proxy,
    :domain_filters,
    :sid_patterns,
    :logout_signatures,
    :active_policy,
    :active_policies,
    :enable_smart_scan
    ]
    project_settings = Hash.new
    settings_names.each do |s|
      project_settings[s] = @settings[s]
    end
    
    #    project_settings = {
    #      :project_path => @settings[:project_path],
    #      :session_path => @settings[:session_path],
    #      :project_name => @settings[:project_name],
    #      :module_path => @settings[:module_path],
    #      :intercept_port => @settings[:intercept_port],
    #      :proxy_list => @settings[:proxy_list],
    #      :proxy => @settings[:proxy],
    #      :domain_filters => @settings[:domain_filters],
    #      :sid_patterns => @settings[:sid_patterns],
    #      :logout_signatures => @settings[:logout_signatures],
    #      :active_policy => @settings[:active_policy],
    #      :active_policies => @settings[:active_policies],
    #    }
    
    Watobo::Utils.save_settings(@project_file, 0, "project_settings", project_settings)
  end
  
  def saveSessionSettings()
    session_settings = Hash.new
    session_settings[:date] = @date 
    session_settings[:excluded_chats] = @excluded_chats
    session_settings[:project_name] = @settings[:project_name]
    session_settings[:session_name] = @settings[:session_name]
    session_settings[:loginscript_ids] = @loginscript_ids
    
    Watobo::Utils.save_settings(@session_file, 0, "session_settings", session_settings)
    
  end
  
  def loadSessionSettings()
    begin
      settings = Watobo::Utils.loadSettings(@session_file)
      if settings then
        @settings[:excluded_chats] = settings[:excluded_chats]
        @settings[:project_name] = settings[:project_name]
        @loginscript_ids = settings[:loginscript_ids]
        return true
      else
        puts "! Could not load session settings from file #{@session_file}"
        return false
      end
    rescue => bang
      puts "!!!ERROR: could not load session settings"
      puts bang
      return false
    end
  end
  
  def loadProjectSettings()
    begin
      
      settings = Watobo::Utils.loadSettings(@project_file)
      if settings then
        @settings.update(settings)
        return true
      else
        puts "! Could not load project settings"
        return false
      end
    rescue => bang
      puts "!!!ERROR: could not update project settings"
      puts bang
      return false
    end
  end
  
  def addFinding(finding_info)
    
    new_finding = Qfinding.new(@findings.length, finding_info)
    
    if not @findings.has_key?(finding_info[:id])
      @findings[finding_info[:id]] = new_finding
      @interface.addFinding(new_finding)
      
      
      begin
        if @findings_path != "" then
          request_file = "#{@findings_path}/#{new_finding.id}-request"
          response_file = "#{@findings_path}/#{new_finding.id}-response"
          finding_file = "#{@findings_path}/#{new_finding.id}-finding"
          if not File.exists?(request_file) then
            fh = File.new(request_file, "w+b")
            fh.print new_finding.request
            fh.close
            
            fh = File.new(response_file, "w+b")
            fh.print new_finding.response
            fh.close
            
            Watobo::Utils.save_settings(finding_file, 0, "@saved_settings", new_finding.info)
            
          end
          #    puts "Create File: #{request_file}"
          #    puts "Create File: #{response_file}"
        end
      rescue => bang
        puts "!!!ERROR: #{Module.nesting[0].name}"
        puts bang
      end
    end
    
  end
  
  #
  # INITIALIZE
  #
  # 
  def initialize(interface, project_settings)
    
    # default settings
    @interface = interface
    @active_checks = true
    @dashboard = nil
    @conversations_path = ""
    
    @settings= Hash.new  
    @settings[:proxy] = ""
    @settings[:intercept_port] = 0
    @settings[:module_path] = ''
    @settings[:domain_filters] = []
    @settings[:max_parallel_checks] = 20
    @settings[:target_sites] = []
    @settings[:enable_smart_scan] = true
    @settings[:progress_window] = nil
    @settings[:ignore_parms] = []        # list of parameter names which will not be used for checks/fuzz/scans
    @settings[:smart_scan_parms] = []     # list of parameters which will not be used for unique hashing of request
    
    @numRunningChecks = 0
    
    t = Time.now        
    @date = t.strftime("%m/%d/%Y@%H:%M:%S")  
    
    @interceptor = nil   
    @intercept_response = false
    @intercept_request = false
    @unique_requests = Hash.new
    
    @sessionMgr = QSessionManager.new(self) 
    
    @settings.update(project_settings)
    
    
    
    
    @conversations_path = File.join(@settings[:session_path], "conversations")
    @findings_path = File.join(@settings[:session_path], "findings")
    @log_path = File.join(@settings[:session_path], "logs")
    
    [ @conversations_path, @findings_path, @log_path ].each do |folder|
      if not File.exists?(folder) then
        puts "create path #{folder}"
        begin
          Dir.mkdir(folder)
        rescue SystemCallError => bang
          puts "!!!ERROR:"
          puts bang
        rescue => bang
          puts "!!!ERROR:"
          puts bang
        end
      end
    end
    
    
    
    # @settings[:module_path] = @settings[:module_path]
    @max_response_size = 50000
    
    @active_modules = []
    @passive_modules = []
    @excluded_sites = []
    @excluded_urls = []
    @excluded_chats = [] # list of chat-ids which will not be used for active checks
    
    # prepare communications
    @chat_list=[]
    @findings = Hash.new
    
    @valid_sids = Hash.new
    
    @loginscript_ids = [] 
    #--------------------------------
    # file extensions
    #--------------------------------
    #@fext_pix=%w[ jpg gif bmp ico png ]
    #@fext_docs=%w[ ppt doc xls pptx docx xlsx pdf ]
    #@fext_javascript=%w[ js ]
    #@fext_style=%w[ css ]
    
    
    @session_file = File.join(@settings[:session_path], @settings[:session_name] + ".wss") 
    
    
    @project_file = File.join(@settings[:project_path], @settings[:project_name] + ".wps")
    
    
    
    puts "Load Project Settings:"
    puts "processing file #{@project_file}"
    if not loadProjectSettings() then
      saveProjectSettings()
    end
    
    puts "Load Session Settings:"
    puts "processing file #{@session_file}"
    if not loadSessionSettings() then
      saveSessionSettings()
    end
    
    importSession()
    # read all existing modules to array
    puts "initialize active modules ..."
    init_active_modules
    puts "aktive modules ok!"
    puts "initialize passive modules ..."
    init_passive_modules
    puts "passive modules ok!"
    # start with running passive checks
    #run_passive_checks()
    # importSession()
  end
  
  
  # private
  # @session_status
  # @com_list
  
  
  def init_passive_modules
    ignore = ['.','..','.svn']
    #	puts "get passive modules from path #{@settings[:module_path]}/passive"
    
    begin
      Dir.chdir("#{@settings[:module_path]}/passive")
      #puts "Current Working Directory: \"#{Dir.getwd}\"" if $debug
    rescue StandardError => bang
      puts bang
      puts "\nError changing into source directory \"#{@settings[:module_path]}/passive\"!"
      exit
    end
    Dir.foreach(".") do |modules|
      begin
        
        if modules =~ /.rb/
          # puts "#{@settings[:module_path]}/passive/#{modules}"
          require "#{@settings[:module_path]}/passive/#{modules}"
          #  puts "+ #{modules}"
          # load "#{@settings[:module_path]}/#{modules}/#{check}"
          classname = modules.slice(0..0).upcase + modules.slice(1..-1).downcase
          classname.sub!(".rb","")
          
          # How to get a class constant out of a string ??? Here we go ...
          puts "initializing passive module #{classname}"
          class_constant = Watobo.class_eval("Watobo::Modules::Passive::#{classname}")
          
          @passive_modules.push class_constant.new(self)
          
        end
      rescue => bang
        puts "!!!"
        puts bang
      end
    end      
  end
  
  def init_active_modules
    ignore = ['.','..','.svn']
    # ignore_modules = ["http_methods.rb"]
    ignore_modules = []
    
    begin
      Dir.chdir("#{@settings[:module_path]}/active")
      #puts "Current Working Directory: \"#{Dir.getwd}\"" if $debug
    rescue StandardError => bang
      puts bang
      puts "\nError changing into source directory \"#{@settings[:module_path]}/active\"!"
      exit
    end
    
    Dir.foreach(".") do |group|
      
      if File.ftype(group) == "directory" and not ignore.include?(group)
        
        Dir.foreach("./#{group}") do |modules|
          
          if File.ftype("./#{group}/#{modules}") != "directory" and not ignore.include?("#{modules}")
            #     Dir.foreach("./#{group}/#{modules}/") do |check|
            if File.ftype("./#{group}/#{modules}") == "file" and modules =~ /.rb/ and not ignore_modules.include?(modules)
              begin
                @interface.log("loading module: /active/#{group}/#{modules}")
                require "#{@settings[:module_path]}/active/#{group}/#{modules}"
                # load "#{@settings[:module_path]}/#{modules}/#{check}"
                group_class = group.slice(0..0).upcase + group.slice(1..-1).downcase
                #
                module_class =modules.slice(0..0).upcase + modules.slice(1..-1).downcase
                module_class.sub!(".rb","")
                # How to get a class constant out of a string ??? Here we go ...
                
                class_constant = Watobo.class_eval("Watobo::Modules::Active::#{group_class}::#{module_class}")
                
                @active_modules.push class_constant.new(self)
                @interface.log("#{modules} loaded.")
                
              rescue => bang
                puts bang
                @interface.log("problems loading module: #{@settings[:module_path]}/active/#{group}/#{modules}")
              end
              
            end
            
          end
        end
      end
      
    end
    
  end
  #
  # S E S S I O N _ I N I T
  #
  def importSession()
    puts "* Import Session:"
    puts "+ Conversation Path: #{@conversations_path}"
    
    puts
    chatIds = listChatIds(@conversations_path, "request")
    findingIds = listChatIds(@findings_path, "request")
    numChats = chatIds.length
    numFindings = findingIds.length
    numImports = numChats + numFindings
    pc = 0
    # pw = @interface.setupProgressWindow("Import Session",0)
    
    chatIds.each do |id|
      begin
        @settings[:progress_window].progress(pc, :total => numImports, :job_name => "Import Conversation", :step_name => "load chat: #{id}") if @settings[:progress_window]
        request, response = Watobo::Utils.loadChat(@conversations_path, id.to_s, "-request", "-response")
        chat = Qchat.new(request, response, id)
        addChat(chat)
        pc += 1
      rescue => bang
        puts "!!!ERROR: Could not import conversations"
        puts bang
      end
    end
    
    
    puts "+ Findings Path: #{@findings_path}"
    findingIds.each do |id|
      begin
        @settings[:progress_window].progress(pc, :total => numImports, :job_name => "Import Findings", :step_name => "load finding: #{id}") if @settings[:progress_window]
        request, response = Watobo::Utils.loadChat(@findings_path, id.to_s, "-request", "-response")
        
        fs = Watobo::Utils.loadSettings(File.join(@findings_path, "#{id}-finding"))
        # puts fs.class.to_s
        fs[:chat] = Qchat.new(request, response, id)
        
        finding = Qfinding.new(id, fs)
        if not @findings.has_key?(finding.info[:id]) then
          @findings[finding.info[:id]] = finding
          @interface.addFinding(finding)
          pc += 1
        end
      rescue => bang
        puts "!!!ERROR: Could not import finding [#{id}]"
        puts bang
      end
    end
    
    #pw.close
  end
  
  
  def listChatIds(path, pattern)
    id_list = []
    Dir.foreach(path) do |file|
      if file =~ /#{pattern}/ then        
        id = file.gsub!(/-#{pattern}/,'').to_i
        id_list.push id     
      end              
    end    
    return id_list.uniq.sort
  end
  
  
  
  def initSessionOld()
    pattern="request"
    filelist=[]
    #orderids=[]
    begin
      Dir.chdir(path)
      #puts "Current Working Directory: \"#{Dir.getwd}\"" if $debug
    rescue StandardError => bang
      puts bang
      puts "\nError changing into source directory \"#{path}\"!"
      exit
    end
    
    # find all files in dir where pattern match
    # setup list of filenames in numerical order
    min=0
    max=0
    fileids=[]
    
    
    begin
      numTotalSteps = Dir.entries(path).length * 2 - 1    
      
      pWindow = @interface.setupProgressWindow("Initialize Project", numTotalSteps)
      
    rescue => bang
      puts "could not create progress window"
      puts bang
    end
    
    pIndex = 0
    
    
    num=min
    while num <= max 
      if fileids.include?(num) then     
        fileids.delete(num)
        fname = num.to_s + "-request"
        filelist.push  fname
        # puts fname    
      end
      num += 1
    end
    
    
    filelist.each do |req_file|      
      begin
        #puts "update progress"
        pWindow.update_progress("Initialize Project: Chat-Init",nil, pIndex)
        pIndex += 1
      rescue => bang
        puts bang
      end
      
      request=[]
      response=[]
      begin
        ##################################
        # read request
        # puts req_file
        reqFH = open(req_file,"r")  
        reqFH.each_line do |l|
          #l.gsub!(/\r?\n?$/,"\r\n")
          # l.chop!
          request.push(l)
        end
      rescue => bang
        puts "! Could not open file: #{req_file}"
        puts bang
      end
      #############################
      # read response
      
      # first only read header as array
      content_length = 0
      response_is_gzipped = false
      content_is_chunked = false
      begin
        res_file = req_file.gsub(/request/,'response')
        resFH = open(res_file,"rb")
        
        loop do
          l = resFH.readline
          if l =~ /Content-Length.* (\d*)/ then
            content_length = $1.to_i
            #puts "Content-Length is #{content_length}"
          end
          if l =~ /Content-Encoding.*gzip/ then
            response_is_gzipped = true
          end
          if l=~ /Transfer-Encoding.*chunked/i then
            content_is_chunked = true
          end
          response.push(l)
          # break if l.length < 3 # end of header
          break if l =~ /^\r\n$/ # end of header
        end    
        
        if content_is_chunked then
          dummy = resFH.readline
          content_length = dummy.chomp.hex
        end
      rescue EOFError
        
      rescue => bang
        puts "!!! Error: Could not read response file #{res_file}"
        puts bang
      end
      
      if response_is_gzipped and content_length > 0 then
        @max_response_size = 50000
        gziped=resFH.read(content_length)
        begin
          gz = Zlib::GzipReader.new( StringIO.new( gziped ) )
          data = gz.read
          if data.length > @max_response_size then
            data = data[0..@max_response_size]
            puts "!!! chat file (#{res_file}: Response too long"
            # puts data
          end
          response.push data
        rescue => bang
          puts "ERROR: GZIP with file #{res_file}"
          puts bang
          #resFH.each do |l|
          #  response.push(l) if response.join.length < @max_response_size
          #end
        end
      else
        resFH.each do |l|
          response.push(l) if response.join.length < @max_response_size
        end
      end
      
      
      #end
      if request.first.gsub(/[^(:\/)]\/.*/,"") =~ /#{@settings[:target_filter]}/
        this_chat=Qchat.new(request,response,req_file.gsub!(/-request/,'').to_i)
        #puts "#{this_chat.request.first} : #{@target_filter}"
        # if this_chat.request.first =~ /#{@target_filter}/
        #  @interface.addRequest(self,this_chat)
        @chat_list.push this_chat
      else
        request.clear
        response.clear
      end
    end
    
    @interface.closeProgressWindow
    
  end
  
  
end
