# .
# qRequest.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'socket'
require 'timeout'
require 'openssl'
require 'optparse'
require 'lib/httpparser.rb'
require 'lib/qShapers.rb'
require 'lib/http_socket.rb'

module Watobo
  module Framework
    
    class Session
      attr :lasterror
      
      attr :check_name
      attr :description
      
      attr :name
      attr :description
      
      
      def enabled?
        @enabled
      end
      
      def enable
        @enabled = true
      end
      
      def disable
        @enable = false
      end
      
      
      def read_response(socket)
        response = []
        begin
          Watobo::Framework::HTTP.read_header(socket) do |line|
            response.push line
          end
        rescue => bang
          puts "!ERROR: read_header"
        end
        response.extend Watobo::Parser::Web10
        
        clen = response.content_length
        data =""
        #   timeout(5) do
        begin
          if  clen > 0           
            Watobo::Framework::HTTP.read_body(socket) do |c|
              data += c
              break if data.length == clen
            end          
          else          
            #   puts "* no content-length information ... mmmmmpf"
            eofcount = 0
            Watobo::Framework::HTTP.read_body(socket) do |c|
              data += c
            end
            
          end
        rescue => e
          puts "!!! ERROR: read_response"
          puts e
        end
        # end
        response.push data
        return response
      end
      
      def update_sids(response)
        valid_sid = ""
        response.each do |line|
          @project.settings[:sid_patterns].each do |pat|
            # puts "check #{pat}"
            
            # if line =~ /(Set-Cookie:|Location:).*(#{pat})/ then 
            if line =~ /#{pat}/i then
              sid_key = $1.upcase
              sid_value = $2
              puts "GOT NEW SID (#{sid_key}): #{sid_value}"
              @project.valid_sids[sid_key] = sid_value
            end
          end
          
        end
      end
      
      # +++ doRequest(request)  +++
      # + function: 
      #
      def doRequest(request, prefs)
        # reset lasterror here
        #  puts request
        
        @lasterror = nil
        response = []
        use_proxy = false
        if @project.settings[:proxy] =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):(\d{1,4})/ then
          print "* using forwarding proxy #{@project.settings[:proxy]}\r\n"
          use_proxy = true
          host = $1
          port = $2
        else
          host = request.host
          port = request.port
        end
        
        # default settings for session handling        
        @session_handling_prefs[:update_session] = true
        @session_handling_prefs[:update_contentlength] = true
        @session_handling_prefs[:update_sids] = false
        
        begin
          @session_handling_prefs.update(prefs) if prefs
          
          if @session_handling_prefs[:update_session] == true then 
            request.each do |line|
              @project.settings[:sid_patterns].each do |pat|
                
                if line =~ /#{pat}/i then  
                  sid_key = $1.upcase
                  old_value = $2                  
                  if @project.valid_sids.include?(sid_key) then
                    if not old_value =~ /#{@project.valid_sids[sid_key]}/ then # sid value has changed and needs update
                      puts "+ update sid #{sid_key}"
                      puts "#{old_value}"
                      puts "#{@project.valid_sids[sid_key]}"
                      puts "---"
                      dummy = Regexp.quote(old_value)
                      res = line.gsub!(/#{dummy}/, @project.valid_sids[sid_key])
                      if not res then puts "!!!could not update sid (#{sid_key})"; end
                      puts "->#{line}"
                    end
                  end
                end
              end
            end
          end
        rescue => bang
          #   raise bang
          @lasterror = "ERROR: Updating SIDs"
          puts "ERROR: Updating SIDs"
          puts bang
          
        end
        
        #---------------------------------------
        request.remove_header("^Proxy-Connection") #if not use_proxy
        request.remove_header("^Connection") #if not use_proxy
        request.remove_header("^Accept-Encoding")
        # If-Modified-Since: Tue, 28 Oct 2008 11:06:43 GMT
        # If-None-Match: W/"3975-1225192003000"
        request.remove_header("^If-")
        
        
        if @session_handling_prefs[:update_contentlength] == true then
          request.fix_content_length()
        end 
        
        
        request.add_header("Connection","Close") #if not use_proxy
        request.add_header("Proxy-Connection","Close") #if not use_proxy
        request.add_header("Accept-Encoding","None") #don't want encoding
        #request.add_header("Via", "Watobo") if use_proxy
        # puts request
        
        
        begin
          tcp_socket = nil
          #  timeout(6) do
          
          tcp_socket = TCPSocket.new( host, port)
          tcp_socket.setsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1)
          tcp_socket.sync = true
          #  end
          
          if request.proto =~ /https/i then
            # NEED SPECIAL HANDSHAKE IF SSL-PROXY IS IN PLACE
            if @project.settings[:proxy] =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):(\d{1,4})/ then
              tcp_socket.print "CONNECT #{request.host}:#{request.port} HTTP/1.0\r\n"+
              # "Proxy-Connection: Keep-Alive\r\n" +
                                 "\r\n"
              while line = tcp_socket.gets   
                break if line.length < 3
              end  
              
            end
            @ctx = OpenSSL::SSL::SSLContext.new('SSLv23_client')
            @ctx.key = nil
            @ctx.cert = nil
            socket = OpenSSL::SSL::SSLSocket.new(tcp_socket, @ctx)
            
            socket.connect
            socket.setsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1)
          else
            socket = tcp_socket
          end
          
          # remove URI before sending request
          uri_cache = nil
          if not @project.settings[:proxy] =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):(\d{1,4})/ then
            uri_cache = request.removeURI              
          end
          
          # puts request          
          data = request.join + "\r\n" 
          #   puts data
          # restore uri
          request.restoreURI(uri_cache)
          #  puts request.first
          
          socket.print data
          #if socket.class.to_s =~ /SSLSocket/
          #  socket.io.shutdown(1)
          #else
          #  socket.shutdown(1)
          #end
          print(".")    
          # read response
          response = read_response(socket)
          socket.close
          #printf("\r\nR<-#{response.join.length} (bytes)\r\n")
          print("o")
          #     end
          # return request,response
        rescue Errno::ECONNREFUSED
          response.push "WATOBO: connection refused (#{host}:#{port})"
          request = nil
        rescue Errno::ECONNRESET
          puts '!!! connection reset'
        rescue Errno::EHOSTUNREACH
          response.push "WATOBO: host unreachable (#{host}:#{port})"
          request = nil
        rescue Timeout::Error
          response.push "WATOBO: TimeOut (#{host}:#{port})"
          response.push request.first if request
          request = nil
          puts "+ RequestError: Timeout +"
        rescue Errno::ETIMEDOUT
          response.push "WATOBO: TimeOut (#{host}:#{port})"
          response.push request.first if request
          request = nil
          puts "+ RequestError: Timeout +"
        rescue EOFError
          #puts "EOF"
          #printf("O\r\n")
        rescue Errno::ENOTCONN
          puts "!!!ENOTCONN"
        rescue SocketError => bang
          response.push "WATOBO: SocketError\r\n"
          response.push bang
          request = nil
        rescue  => bang
          puts "Exception!! in Module #{Module.nesting[0].name}"
          puts bang
          # puts bang.backtrace.join("\n")
          @lasterror = bang
          # raise
          # ensure
        end
        if @session_handling_prefs[:update_sids] then
          update_sids(response)
        end
        response.extend Watobo::Parser::Web10
        #socket.shutdown(2)          
        return request, response
        #end
        
      end
      
      def initialize(project)
        @project=project
        @lasterror=""
        @session_handling_prefs = Hash.new
        @session_handling_prefs[:update_session] = true 
        @session_handling_prefs[:update_contentlength] = true 
        @session_handling_prefs[:update_sids] = false
        @enabled = true
        @check_name="undefined"
        @description="undefined"
        @socket = nil
        
        @ctx = nil
        # TODO: Implement switches for URL-Encoding (http://www.blooberry.com/indexdot/html/topics/urlencoding.htm)
        # TODO: Implement switches for Following Redirects
        # TODO: Implement switches for Logging, Debugging, ...
      end
    end
    
  end
end