# .
# qShapers.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/core/watobo_constants'

module Watobo
  module Framework
    module Shaper
      module Web10
        
        def replace_post_parm(parm,value)
          parm_quoted = Regexp.quote(parm)
          self.last.gsub!(/([?&]{0,1})#{parm_quoted}=([0-9a-zA-Z\-\._,+<>\%!=]*)(&{0,1})/i, "\\1#{parm}=#{value}\\3")
        end
        
        def replace_get_parm(parm,value)
          parm_quoted = Regexp.quote(parm)
          # puts "replacing parameter #{parm} with value #{value}"
          # self.first.gsub!(/([?&]{0,1})#{parm}=([0-9a-zA-Z\-\._,+<>\%!=]*)(&{0,1})/i, "\\1#{parm}=#{value}\\3")
          self.first.gsub!(/([?&]{1})#{parm_quoted}=([^ &]*)(&{0,1})/i, "\\1#{parm}=#{value}\\3")
        end
        
        def replace_http_method(new_method)
          self.first.gsub!(/^\w*/i, "#{new_method}")
        end
        
        def replace_element(new_element)
          #   puts "replace element #{new_element}"
          new_element.gsub!(/^\//, "")
          self.first.gsub!(/(.*\/)(.*) (HTTP.*)/i,"\\1#{new_element} \\3")
        end
        
        def add_post_parm(parm,value)
          
        end
        
        def add_get_parm(parm,value)
          line = self.shift
          new_p = "&"
          new_p = "?" if not self.element =~ /\?/
          new_p += parm
          line.gsub!(/( HTTP\/.*)/, "#{new_p}=#{value}\\1")
          self.unshift(line)
        end
        
        def add_header(header, value)
          self_copy = []
          self_copy.concat(self.headers)
          self_copy.push "#{header}: #{value}\r\n"
          
          if self.body.length > 0 then
            self_copy.push "\r\n"
            self_copy.concat(self.body)
          end
          
          self.replace(self_copy)
          
        end
        
        def removeURI
          if self.first =~ /^(.*)(https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)/ then
            uri = $2
            self.first.gsub!(/^(.*)(#{Regexp.quote(uri)})/,"\\1/")
            
            return uri
          else
            return nil
          end
          #self.first.gsub!(/^(.*)(https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)/,"\\1/")
        end
        
        def restoreURI(uri)
          if self.first =~ /^(\w+) \/(.*)/ then
            method = $1
            rest = $2
            self.first.gsub!(/^\w.*/, "#{method} #{uri}#{rest}")
            
          else
            return nil
          end
          #self.first.gsub!(/^(.*)(https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)/,"\\1/")
        end
        
        #
        # R E M O V E _ H E A D E R
        #
        def remove_header(header)
          self_copy = []
          eoh = false
          self.each do |line|
            # puts "+ + + #{line}"
            if not eoh == true then
              if not line =~ /#{header}/i
                self_copy.push line
              end  
            else
              self_copy.push line
            end
            
            if line.strip.length == 0 then
              #  puts "header finished"
              eoh = true 
            end
          end
          self.replace(self_copy)
        end
        
        def replace_header(header, value)
          
        end
        
        def replace_method(method)
          
        end
        
        def fix_session(pattern,value)
          
        end
        
        def fix_content_length
          self_copy = []
          self.each do |line|
            if line =~ /^Content-Length/i
              #  puts "--- fix content-length ---"
              #  puts line
              #  puts self.body.join.length.to_s
              #  puts "---"
              self_copy.push "Content-Length: #{self.body.join.length.to_s}\r\n"
              #self_copy.push "Content-Length: #{self.body.join("\r\n").length.to_s}\r\n"
            else
              self_copy.push line
            end         
          end
          self.replace(self_copy)
        end
        
        def fixupContentLength
          te = self.transferEncoding
          if te == Watobo::Core::TE_CHUNKED then
            puts "Transfer-Encoding = TE_CHUNKED"
            self.remove_header("Transfer-Encoding")
            self.add_header("Content-Length", "0")
            new = []
            new.concat self.headers
            new.push "\r\n"
            
            dummy = self.body.join.split(/\n/)
            
            chunk = dummy.first.strip.hex
            dummy.shift
            bytes_to_read = chunk
            line_count = 0
            dummy.each{ |e|
            line_count += 1
            if e.strip =~ /^[0-9a-fA-F]*$/ then
              # don't know exactly if hexvalues are chunk ... mmmhh
              # so the maximum difference should be line_count * 2 (\r\n)
               if bytes_to_read >= line_count*2 then
                 new.push "#{e}\n"
                 bytes_to_read -= e.length
               else
                 bytes_to_read = e.strip.hex
                 line_count = 0
               end
            else
              new.push "#{e}\n" 
              bytes_to_read -= e.length
            end
            }
            
  #          chunk = dummy.shift.hex
  #          puts "Chunk-Size: #{chunk.to_s}"
  #          i = 0
  #          while chunk > 0 
  #          puts "Chunk-Size: #{chunk.to_s}"
  #            c = 0
  #            while c < chunk and not dummy[i].strip =~ /[0..9a..fA..F]*/
  #              c += dummy[i].length
  #              new.push dummy[i]
  #              i += 1
  #            end
              #chunk = dummy[i].hex
 #             puts dummy[i]
 #             puts chunk.to_s
 #             i += 1
 #             puts i.to_s
 #           end
            self.replace(new)
            self.fix_content_length
            
          elsif te == Watobo::Core::TE_NONE then
            self.fix_content_length
          end
          
        end
        
        
        def setHTTPVersion(version)
          self.first.gsub!(/HTTP\/(.*)$/, "HTTP\/#{version}")
          puts "HTTPVersion fixed: #{self.first}"
        end
        
      end
    
    
    end
  end
end
