# .
# load_chat.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Utils
    def Utils.loadChat(path, id, request_pattern, response_pattern)
      chat = nil
      req_file = File.join(path, "#{id}#{request_pattern}")
      res_file = File.join(path, "#{id}#{response_pattern}")
   #   puts "* load chat (request): #{req_file}"
   #   puts "* load chat (response): #{res_file}"
      request = []
      if File.exists?(req_file) then
        fh = File.open(req_file,"r")
        fh.each do |line|  
          request.push line
        end
      else
        puts "!! File not found (#{req_file})"
        return nil
      end
      
      response = []
     
      # first only read header as array
      content_length = 0
      response_is_gzipped = false
      content_is_chunked = false
      max_response_size = 50000
      
      if File.exists?(res_file) then
        begin
          
          resFH = open(res_file, "rb")
          
          loop do
            l = resFH.readline
            if l =~ /Content-Length.* (\d*)/ then
              content_length = $1.to_i
              #puts "Content-Length is #{content_length}"
            end
            if l =~ /Content-Encoding.*gzip/ then
              response_is_gzipped = true
            end
            if l=~ /Transfer-Encoding.*chunked/i then
              content_is_chunked = true
            end
            response.push(l)
            # break if l.length < 3 # end of header
            break if l =~ /^\r\n$/ # end of header
          end    
          
          if content_is_chunked then
            # read rest of file
            response.push "\r\n"
            response.push resFH.read
            #content_length = dummy.chomp.hex
            return request, response
          end
       rescue => bang
         puts "Could not read Header from file #{res_file}"
       end
       # now read response body
       begin
          if response_is_gzipped and content_length > 0 then
            
            gziped = resFH.read(content_length)
            begin
              gz = Zlib::GzipReader.new( StringIO.new( gziped ) )
              data = gz.read
              if data.length > max_response_size then
                data = data[0..max_response_size]
                puts "!!! chat file (#{res_file}: Response too long"
                # puts data
              end
              response.push data
            rescue => bang
              puts "ERROR: GZIP with file #{res_file}"
              puts bang
              #resFH.each do |l|
              #  response.push(l) if response.join.length < @max_response_size
              #end
            end
            return request, response
          end
            resFH.each do |l|
              response.push(l) if response.join.length < max_response_size
            end
          return request, response
        rescue EOFError => bang
          return request, response
        rescue => bang
          puts "!!! Error: Could not read response file #{res_file}"
          puts bang
        end
        
      end
    end
    
    
  end
end